@@ -24,14 +24,17 @@
 
 #include "koshell_shell.h"
 
+#include <ktempfile.h>
 #include <kfiledialog.h>
 #include <klocale.h>
 #include <kdebug.h>
-#include <kstddirs.h>
+#include <kstandarddirs.h>
 #include <klibloader.h>
 #include <kservice.h>
 #include <kmessagebox.h>
 #include <krecentdocument.h>
+#include <kparts/partmanager.h>
+#include <kaction.h>
 
 #include <koQueryTrader.h>
 #include <koKoolBar.h>
@@ -40,30 +43,34 @@
 #include <koView.h>
 #include <koPartSelectDia.h>
 #include <koFilterManager.h>
+#include <kiconloader.h>
 
 KoShellWindow::KoShellWindow()
  : KoMainWindow( KGlobal::instance() )
 {
   m_activePage = m_lstPages.end();
 
-  m_pLayout = new QHBox( this );
+  m_pLayout = new QHBox( centralWidget() );
 
   m_pKoolBar = new KoKoolBar( m_pLayout );
 
-  setCentralWidget( m_pLayout );
-
   m_pFrame = new KoShellFrame( m_pLayout );
 
   m_grpFile = m_pKoolBar->insertGroup(i18n("Parts"));
-  QValueList<KoDocumentEntry> lstComponents = KoDocumentEntry::query();
+  QValueList<KoDocumentEntry> lstComponents = KoDocumentEntry::query(false,QString());
   QValueList<KoDocumentEntry>::Iterator it = lstComponents.begin();
   for( ; it != lstComponents.end(); ++it )
   {
-      //kdDebug() << "Inserting into koolbar : " << (*it).name << endl;
-      int id = m_pKoolBar->insertItem( m_grpFile,
+      //kdDebug() << "Inserting into koolbar : " << (*it).name() << endl;
+      int id;
+      if (!(*it).service()->genericName().isEmpty()) //skip the unavailable part
+          id = m_pKoolBar->insertItem( m_grpFile,
                                        DesktopIcon((*it).service()->icon()),
                                        (*it).name(),
                                        this, SLOT( slotKoolBar( int, int ) ) );
+      else
+          continue;
+
       m_mapComponents[ id ] = *it;
 
       // Build list of patterns for all supported KOffice documents...
@@ -87,15 +94,26 @@ KoShellWindow::KoShellWindow()
   m_pKoolBar->setFixedWidth( 80 );
   m_pKoolBar->setMinimumHeight( 300 );
 
+  connect( this, SIGNAL( documentSaved() ),
+           this, SLOT( slotNewDocumentName() ) );
+
   // Not implemented yet
   actionCollection()->action("view_split")->setEnabled(false);
   actionCollection()->action("view_splitter_orientation")->setEnabled(false);
+
+  m_client = new KoShellGUIClient( this );
+  createShellGUI();
 }
 
 KoShellWindow::~KoShellWindow()
 {
   //kdDebug() << "KoShellWindow::~KoShellWindow()" << endl;
 
+  // Set the active part to 0 (as we do in ~KoMainWindow, but this is too
+  // late for KoShell, it gets activePartChanged signals delivered to a dead
+  // KoShellWindow object).
+  partManager()->setActivePart(0);
+
   // Destroy all documents - queryClose has made sure we saved them first
   QValueList<Page>::ConstIterator it = m_lstPages.begin();
   for (; it != m_lstPages.end(); ++it )
@@ -107,48 +125,97 @@ KoShellWindow::~KoShellWindow()
   }
   m_lstPages.clear();
 
-  setRootDocumentDirect( 0L, QList<KoView>() ); // prevent our parent destructor from doing stupid things
+  setRootDocumentDirect( 0L, QPtrList<KoView>() ); // prevent our parent destructor from doing stupid things
 }
 
-QString KoShellWindow::nativeFormatName() const
-{
-  return i18n("KOffice Documents");
-}
-
-QString KoShellWindow::nativeFormatPattern() const
-{
-  return m_filter; // "*.kwd *.ksp *.kpr";
-}
-
-bool KoShellWindow::openDocument( const KURL & url )
-{
-    return openDocumentInternalKoShell( 0L, url );
-}
-
-// Should be openDocumentInternal if it was virtual in KoMainWindow
-bool KoShellWindow::openDocumentInternalKoShell( KoFilterManager * filterManager, const KURL &url )
+bool KoShellWindow::openDocumentInternal( const KURL &url, KoDocument* )
 {
+  // Here we have to distinguish two cases: The passed URL has a native
+  // KOffice mimetype. Then we query the trader and create the document.
+  // The file is loaded and everyone's happy.
+  // The second case is a non-native file. Here we have to create a
+  // filter manager, ask it to convert the file to the "closest" available
+  // KOffice part and open the temporary file.
   KMimeType::Ptr mimeType = KMimeType::findByURL( url );
   m_documentEntry = KoDocumentEntry::queryByMimeType( mimeType->name().latin1() );
-  if ( m_documentEntry.isEmpty() )
-    return false;
 
-  m_recent->addURL( url );
+  KTempFile* tmpFile = 0;
+  KURL tmpUrl( url );  // we might have to load a converted temp. file
+
+  if ( m_documentEntry.isEmpty() ) { // non-native
+    tmpFile = new KTempFile;
+
+    KoFilterManager *manager = new KoFilterManager( url.path() );
+    QCString mimetype;                                               // an empty mimetype means, that the "nearest"
+    KoFilter::ConversionStatus status = manager->exp0rt( tmpFile->name(), mimetype ); // KOffice part will be chosen
+    delete manager;
+
+    if ( status != KoFilter::OK || mimetype.isEmpty() ) {
+      tmpFile->unlink();
+      delete tmpFile;
+      return false;
+    }
+
+    // If the conversion was successful we get the mimetype of the
+    // chosen KOffice part back.
+    m_documentEntry = KoDocumentEntry::queryByMimeType( mimetype );
+    if ( m_documentEntry.isEmpty() ) {
+      tmpFile->unlink();
+      delete tmpFile;
+      return false;
+    }
+
+    // Open the temporary file
+    tmpUrl.setPath( tmpFile->name() );
+  }
+
+  recentAction()->addURL( url );
 
   KoDocument* newdoc = m_documentEntry.createDoc();
 
-  // Pass the filterManager to the document (who will own it from now on)
-  if ( filterManager )
-      newdoc->setFilterManager( filterManager );
-
   connect(newdoc, SIGNAL(sigProgress(int)), this, SLOT(slotProgress(int)));
   connect(newdoc, SIGNAL(completed()), this, SLOT(slotKSLoadCompleted()));
   connect(newdoc, SIGNAL(canceled( const QString & )), this, SLOT(slotKSLoadCanceled( const QString & )));
-  if ( !newdoc || !newdoc->openURL( url ) )
+  newdoc->addShell( this ); // used by openURL
+  bool openRet = (!isImporting ()) ? newdoc->openURL(tmpUrl) : newdoc->import(tmpUrl);
+  if ( !newdoc || !openRet )
   {
       delete newdoc;
+      if ( tmpFile ) {
+        tmpFile->unlink();
+        delete tmpFile;
+      }
       return false;
   }
+
+  if ( tmpFile ) {
+    //if the laoded file has been a temporary file
+    //we need to correct a few document settings
+    //see description of bug #77574 for additional information
+  
+    //correct (output) mime type: we need to set it to the non-native format
+    //to make sure the user knows about saving to a non-native mime type
+    //setConfirmNonNativeSave is set to true below
+    newdoc->setMimeType( mimeType->name().latin1() );
+    newdoc->setOutputMimeType( mimeType->name().latin1() );
+    
+    //the next time the user saves the document he should be warned
+    //because of mime type settings done above;
+    newdoc->setConfirmNonNativeSave(true,true); //exporting,warn_on
+    newdoc->setConfirmNonNativeSave(false,true); //save/save as,warn_on
+    
+    //correct document file (should point to URL)
+    newdoc->setFile( url.path() );
+    
+    //correct document URL
+    newdoc->setURL( url );
+    
+    //update caption to represent the correct URL in the window titlebar
+    updateCaption();
+  
+    tmpFile->unlink();
+    delete tmpFile;
+  }
   return true;
 }
 
@@ -156,6 +223,12 @@ bool KoShellWindow::openDocumentInternalKoShell( KoFilterManager * filterManager
 void KoShellWindow::slotKSLoadCompleted()
 {
     KoDocument* newdoc = (KoDocument *)(sender());
+
+    // KoDocument::import() calls resetURL() too late...
+    // ...setRootDocument will show the URL...
+    // So let's stop this from happening and the user will never know :)
+    if (isImporting()) newdoc->resetURL ();
+
     partManager()->addPart( newdoc, false );
     setRootDocument( newdoc );
     disconnect(newdoc, SIGNAL(sigProgress(int)), this, SLOT(slotProgress(int)));
@@ -167,6 +240,7 @@ void KoShellWindow::slotKSLoadCanceled( const QString & errMsg )
 {
     KMessageBox::error( this, errMsg );
     // ... can't delete the document, it's the one who emitted the signal...
+    // ###### FIXME: This can be done in 3.0 with deleteLater, I assume (Werner)
 
     KoDocument* newdoc = (KoDocument *)(sender());
     disconnect(newdoc, SIGNAL(sigProgress(int)), this, SLOT(slotProgress(int)));
@@ -184,9 +258,10 @@ void KoShellWindow::setRootDocument( KoDocument * doc )
 
   if ( doc )
   {
-    doc->addShell( this );
+    if ( !doc->shells().contains( this ) )
+        doc->addShell( this );
     KoView *v = doc->createView( m_pFrame );
-    QList<KoView> views;
+    QPtrList<KoView> views;
     views.append(v);
     setRootDocumentDirect( doc, views );
 
@@ -200,7 +275,7 @@ void KoShellWindow::setRootDocument( KoDocument * doc )
     page.m_pView = v;
     page.m_id = m_pKoolBar->insertItem( m_grpDocuments,
                                         DesktopIcon( m_documentEntry.service()->icon() ),
-                                        i18n("No name"),
+                                        i18n("Untitled"),
                                         this, SLOT( slotKoolBar( int, int ) ) );
     kdDebug() << " New page has id " << page.m_id << " doc is " << doc << endl;
 
@@ -209,12 +284,17 @@ void KoShellWindow::setRootDocument( KoDocument * doc )
     switchToPage( m_lstPages.fromLast() );
   } else
   {
-    setRootDocumentDirect( 0L, QList<KoView>() );
+    setRootDocumentDirect( 0L, QPtrList<KoView>() );
     m_activePage = m_lstPages.end();
     KoMainWindow::updateCaption();
   }
 }
 
+void KoShellWindow::slotNewDocumentName()
+{
+	updateCaption();
+}
+
 void KoShellWindow::updateCaption()
 {
     kdDebug() << "KoShellWindow::updateCaption() rootDoc=" << rootDocument() << endl;
@@ -231,23 +311,20 @@ void KoShellWindow::updateCaption()
         QString name;
         if ( rootDocument()->documentInfo() )
         {
-            KoDocumentInfoPage * page = rootDocument()->documentInfo()->page( QString::fromLatin1("about"));
-            if (page)
-                name = static_cast<KoDocumentInfoAbout *>(page)->title();
+            name = rootDocument()->documentInfo()->title();
         }
         if ( name.isEmpty() )
-          // Fall back to document URL
-          name = rootDocument()->url().fileName();
+            // Fall back to document URL
+            name = rootDocument()->url().fileName();
 
-        if ( !name.isEmpty() ) // else keep No name
+        if ( !name.isEmpty() ) // else keep Untitled
         {
-          if ( name.length() > 20 )
-          {
-            name.truncate( 17 );
-            name += "...";
-          }
-
-          m_pKoolBar->renameItem( m_grpDocuments, (*it).m_id, name );
+	    if ( name.length() > 20 )
+	    {
+	        name.truncate( 17 );
+	        name += "...";
+	    }
+            m_pKoolBar->renameItem( m_grpDocuments, (*it).m_id, name );
         }
 
         return;
@@ -267,6 +344,9 @@ void KoShellWindow::slotKoolBar( int _grp, int _item )
     KoDocument *doc = m_documentEntry.createDoc();
     if (doc)
     {
+        // koshell isn't starting, but this is like starting a new app:
+        // offer both "open existing file" and "open new file".
+        doc->setInitDocFlags( KoDocument::InitDocAppStarting );
         if ( doc->initDoc() )
         {
             partManager()->addPart( doc, false );
@@ -296,6 +376,14 @@ void KoShellWindow::slotKoolBar( int _grp, int _item )
   }
 }
 
+void KoShellWindow::slotShowSidebar()
+{
+	if( m_pKoolBar->isShown() )
+		m_pKoolBar->hide();
+	else
+		m_pKoolBar->show();
+}
+
 void KoShellWindow::switchToPage( QValueList<Page>::Iterator it )
 {
   // Move away current page (view)
@@ -311,11 +399,12 @@ void KoShellWindow::switchToPage( QValueList<Page>::Iterator it )
   // Make it active (GUI etc.)
   partManager()->setActivePart( (*m_activePage).m_pDoc, v );
   // Change current document
-  QList<KoView> views;
+  QPtrList<KoView> views;
   views.append(v);
   setRootDocumentDirect( (*m_activePage).m_pDoc, views );
-  // Fix caption
+  // Fix caption and set focus to the new view
   updateCaption();
+  v->setFocus();
 }
 
 void KoShellWindow::slotFileNew()
@@ -324,7 +413,10 @@ void KoShellWindow::slotFileNew()
     if ( m_documentEntry.isEmpty() )
       return;
     KoDocument* newdoc = m_documentEntry.createDoc();
-    if ( !newdoc || !newdoc->initDoc() )
+    if ( !newdoc )
+        return;
+    newdoc->setInitDocFlags( KoDocument::InitDocFileNew );
+    if ( !newdoc->initDoc() )
     {
       delete newdoc;
       return;
@@ -334,40 +426,32 @@ void KoShellWindow::slotFileNew()
     setRootDocument( newdoc );
 }
 
-// ### This is an ugly copy of KoMainWindow::slotFileOpen, due to the fact
-// that openDocumentInternal isn't virtual
 void KoShellWindow::slotFileOpen()
 {
     KFileDialog *dialog=new KFileDialog(QString::null, QString::null, 0L, "file dialog", true);
-    dialog->setCaption( i18n("Open document") );
-    KoFilterManager * filterManager = new KoFilterManager;
-    filterManager->prepareDialog(dialog, KoFilterManager::Import,
-                                 KoDocument::readNativeFormatMimeType(),
-                                 nativeFormatPattern(), nativeFormatName(), true);
+    if (!isImporting())
+        dialog->setCaption( i18n("Open Document") );
+    else
+        dialog->setCaption( i18n("Import Document") );
+    dialog->setMimeFilter( KoFilterManager::mimeFilter() );
+
     KURL url;
     if(dialog->exec()==QDialog::Accepted) {
         url=dialog->selectedURL();
-        m_recent->addURL( url );
+        recentAction()->addURL( url );
         if ( url.isLocalFile() )
             KRecentDocument::add(url.path(-1));
         else
             KRecentDocument::add(url.url(-1), true);
     }
     else
-    {
-        delete filterManager;
         return;
-    }
 
-    filterManager->cleanUp();
     delete dialog;
     if ( url.isEmpty() )
-    {
-        delete filterManager;
         return;
-    }
 
-    (void) openDocumentInternalKoShell( filterManager, url );
+    (void) openDocumentInternal( url );
 }
 
 void KoShellWindow::slotFileClose()
@@ -426,23 +510,23 @@ void KoShellWindow::closeDocument()
 bool KoShellWindow::queryClose()
 {
   // Save current doc and views
-  QList<KoView> currentViews;
+  QPtrList<KoView> currentViews;
   KoDocument * currentDoc = 0L;
   bool ok = true;
   if (m_activePage != m_lstPages.end())
   {
       currentDoc = (*m_activePage).m_pDoc;
       currentViews.append((*m_activePage).m_pView);
-  
+
       // This one is called by slotFileQuit and by the X button.
       // We have to check for unsaved docs...
-      
+
       QValueList<Page>::Iterator it = m_lstPages.begin();
       for( ; it != m_lstPages.end(); ++it )
       {
           // This is quite a HACK
           // We should ask ourselves, to get a better dialog box
-          setRootDocumentDirect( (*it).m_pDoc, QList<KoView>() );
+          setRootDocumentDirect( (*it).m_pDoc, QPtrList<KoView>() );
           // Test if we can close this doc
           if ( !KoMainWindow::queryClose() )
           {
@@ -451,14 +535,14 @@ bool KoShellWindow::queryClose()
           }
       }
 
-      // Restore current doc and views
-      setRootDocumentDirect( currentDoc, currentViews );
+  // Restore current doc and views
+  setRootDocumentDirect( currentDoc, currentViews );
   }
   return ok;
 }
 
 /*
-// Should this be an additionnal action in the File menu ?
+// Should this be an additional action in the File menu ?
 bool KoShellWindow::saveAllPages()
 {
   // TODO
@@ -481,6 +565,11 @@ void KoShellWindow::slotFilePrint()
 }
 */
 
+void KoShellWindow::createShellGUI()
+{
+	guiFactory()->addClient( m_client );
+}
+
 ///////
 
 KoShellFrame::KoShellFrame( QWidget *parent )
@@ -500,4 +589,13 @@ void KoShellFrame::resizeEvent( QResizeEvent * )
     m_pView->setGeometry( 0, 0, width(), height() );
 }
 
+KoShellGUIClient::KoShellGUIClient( KoShellWindow *window ) : KXMLGUIClient()
+{
+	setXMLFile( "koshellui.rc", true, false );
+
+	sidebar = new KToggleAction(i18n("Show Sidebar"), "view_choose", 0, window,
+			SLOT( slotShowSidebar() ), actionCollection(), "show_sidebar");
+	sidebar->setChecked( true );
+}
+
 #include "koshell_shell.moc"
