@@ -3,22 +3,25 @@
 
 #include <qlineedit.h>
 #include <qtooltip.h>
+#include <qpen.h>
 
 #include <koQueryTrader.h>
 #include <kspread_view.h>
+#include <kspread_doc.h>
+#include "kspread_selection.h"
 
 class KSpreadEditWidget;
 class KSpreadCanvas;
 class KSpreadHBorder;
 class KSpreadVBorder;
-class KSpreadTable;
+class KSpreadSheet;
 class KSpreadDoc;
 class KSpreadPoint;
 class KSpreadRange;
 class KSpreadCellEditor;
 class KSpreadCell;
 class QWidget;
-
+class QTimer;
 
 class QPainter;
 class QLabel;
@@ -100,6 +103,7 @@ public:
     enum EditorType { CellEditor, FormulaEditor, EditWidget };
 
     KSpreadCanvas( QWidget *_parent, KSpreadView *_view, KSpreadDoc* _doc );
+    ~KSpreadCanvas( );
 
     /**
      * Called from @ref KSpreadView to complete the construction. Has to
@@ -107,110 +111,101 @@ public:
      */
     void init();
 
-    KSpreadCellEditor* editor() { return m_pEditor ; }
+    KSpreadCellEditor* editor() const { return m_pEditor ; }
 
-    // ###### Torben: Many of these functions are not used or can be made private
-    QPoint chooseMarker() const { return QPoint( m_i_chooseMarkerColumn, m_i_chooseMarkerRow ); }
-    int chooseMarkerColumn() const { return m_i_chooseMarkerColumn; }
-    int chooseMarkerRow() const { return m_i_chooseMarkerRow; }
-    /**
-     * Sets the marker and redraws it if it is visible.
-     * It does not scroll the canvas. If you want to do that, use
-     * @ref #chooseGotoLocation.
-     *
-     * In addition it does not handel selection
-     * of multiple cells. After calling setChooseMarker() the selection will
-     * consist of the single cell given by @p p.
-     */
-    void setChooseMarker( const QPoint& p );
-    /**
-     * Internal. DONT USE.
-     */
-    void setChooseMarkerColumn( int _c ) {  m_i_chooseMarkerColumn = _c; }
-    /**
-     * Internal. DONT USE.
-     */
-    void setChooseMarkerRow( int _r ) {  m_i_chooseMarkerRow = _r;  }
-    /**
-     * Move the choose selection. That may include switching the table.
-     * The canvas is scrolled to the appropriate position if needed.
-     *
-     * @param make_select determines wether this move if the marker is part of a
-     *                    selection, that means: The user holds the shift key and
-     *                    moves the cursor keys.
-     */
-    void chooseGotoLocation( int x, int y, KSpreadTable* table = 0, bool make_select = FALSE );
-    /**
-     * Internal. DONT USE.
-     */
-    void showChooseMarker(){if(choose_visible==true) {return;}
-    			drawChooseMarker();choose_visible=true;}
-    /**
-     * Internal. DONT USE.
-     */
-    void hideChooseMarker(){if(choose_visible==false) {return;}
-    			drawChooseMarker();choose_visible=false;}
-    /**
-     * Internal. DONT USE.
-     */
-    bool isChooseMarkerVisible() const { return choose_visible; }
 
-    /**
-     * Called from @ref KSpreadView::slotChangeChooseSelection to
-     * draw the selection with a rubber band.
-     */
-    void updateChooseMarker( const QRect& _old, const QRect& _new );
-    /**
-     * If the user chooses some cells during editing a formular, then
+  /**
+     * If the user chooses some cells during editing a formula, then
      * this function returns the length of the textual representation.
-     * For example the user selects "Table1!A1:B2" then this function
+     * For example the user selects "Sheet1!A1:B2" then this function
      * returns 12.
      */
     int chooseTextLen() const { return length_namecell; }
 
+    KSpreadSelection* selectionInfo() const {return m_pView->selectionInfo();}
+    QRect selection() const;
     QPoint marker() const;
     int markerColumn() const;
     int markerRow() const;
 
     void updateCellRect( const QRect &_rect );
-    void updateSelection( const QRect& _old, const QRect& _old_marker );
+    void updateSelection( const QRect& oldSelection, const QPoint& oldMarker );
 
-    const QPen& defaultGridPen() { return m_defaultGridPen; }
+    const QPen& defaultGridPen() const { return m_defaultGridPen; }
 
-    double zoom() { return m_pView->zoom(); }
+    double zoom() const { return m_pView->zoom(); }
 
-    int xOffset() { return m_iXOffset; }
-    int yOffset() { return m_iYOffset; }
+    /**
+     * Returns the width of the columns before the current screen
+     */
+    double xOffset() const { return m_dXOffset; }
+    /**
+     * Returns the height of the rows before the current screen
+     */
+    double yOffset() const { return m_dYOffset; }
 
-    const KSpreadTable* activeTable() const;
-    KSpreadTable* activeTable();
-    KSpreadTable* findTable( const QString& _name );
+    /**
+     * Return a rect indicating which cell range is currently visible onscreen
+     */
+    QRect visibleCells();
+
+    KSpreadSheet* activeTable() const;
+    KSpreadSheet* findTable( const QString& _name ) const;
 
     /**
      * A convenience function.
      */
-    void gotoLocation( const KSpreadRange & _range );
+    bool gotoLocation( const KSpreadRange & _range );
     /**
      * A convenience function.
      */
-    void gotoLocation( const KSpreadPoint& _cell );
+    bool gotoLocation( const KSpreadPoint& _cell );
+
     /**
      * Move the cursor to the specified cell. This may include switching
      * the table. In addition @ref #KSpreadView::updateEditWidget is called.
      *
-     * @param make_select determines wether this move of the marker is part of a
-     *                    selection, that means: The user holds the shift key and
-     *                    moves the cursor keys. In this case the selection is
-     *                    updated accordingly.
-     */
-    void gotoLocation( int x, int y, KSpreadTable* table = 0, bool make_select = FALSE,bool move_into_area=false,bool keypress=false);
+     * @param location the cell to move to
+     *
+     * @param table the table to move to.  If NULL, the active table is used
 
+     * @param extendSelection determines wether this move of the marker is part
+     *                        of a selection, that means: The user holds the
+     *                        shift key and moves the cursor keys. In this case
+     *                        the selection is updated accordingly.
+     *                        If this is false, the cell will be the single cell
+     *                        selected, and the selection anchor will be reset
+     *                        to this cell.
+     */
+    void gotoLocation( QPoint const & location, KSpreadSheet* table = NULL,
+                       bool extendSelection = false);
+
+    /**
+     * convenience function
+     */
+    void gotoLocation( int col, int row, KSpreadSheet* table = NULL,
+                       bool extendSelection = false)
+    {gotoLocation(QPoint(col, row), table, extendSelection);}
+
+
+    /**
+     * Paint all visible cells that have a paint dirty flag set
+     */
+    void paintUpdates();
+
+
+    /**
+     * Makes sure a cell is visible onscreen by scrolling up/down and left/right
+     *
+     * @param location the cell coordinates to scroll to
+     */
+    void scrollToCell(QPoint location);
     /**
      * Chooses the correct @ref #EditorType by looking at
      * the current cells value. By default CellEditor is chosen.
      */
     void createEditor();
-    void createEditor( EditorType type );
+    bool createEditor( EditorType type, bool addFocus = true );
     /**
      * Deletes the current cell editor.
      *
@@ -223,7 +218,7 @@ public:
 
     /**
      * Called from @ref KSpreadEditWidget and KSpreadCellEditor
-     * if they loose the focus becuase the user started a "choose selection".
+     * if they loose the focus because the user started a "choose selection".
      * This is done because the editor wants to get its focus back afterwards.
      * But somehow KSpreadCanvas must know whether the EditWidget or the CellEditor
      * lost the focus when the user clicked on the canvas.
@@ -231,16 +226,18 @@ public:
     void setLastEditorWithFocus( EditorType type ) { m_focusEditorType = type; }
 
     /**
-     * Switches to choose mode and sets the inital selection to the
+     * Switches to choose mode and sets the initial selection to the
      * position returned by @ref #marker.
      */
     void startChoose();
     /**
-     * Switches to choose mode and sets the inital @p selection.
+     * Switches to choose mode and sets the initial @p selection.
      */
     void startChoose( const QRect& selection );
     void endChoose();
 
+    bool chooseMode(){ return m_bChoose; }
+
     /**
     * Adjust a area in height and width
     */
@@ -257,19 +254,27 @@ public:
     // but is in fact a sibling of the canvas, which needs to know about it.
     void setEditWidget( KSpreadEditWidget * ew ) { m_pEditWidget = ew; }
 
-    KSpreadView* view() { return m_pView; }
-    KSpreadDoc* doc() { return m_pDoc; }
+    KSpreadView* view()const { return m_pView; }
+    KSpreadDoc* doc()const { return m_pDoc; }
+
+    virtual bool focusNextPrevChild( bool );
+
+    bool chooseFormulaArea() const { return m_bChoose;}
 
     /**
-     * Find support.
+     * Depending on the offset in "zoomed" screen pixels
+     * for the horizontal direction,
+     * the function returns the steps in unzoomed points
+     * for the autoscroll acceleration
      */
-    void find( const QPoint &_marker, QString _find, long options );
-
+    double autoScrollAccelerationX( int offset );
     /**
-     * Find'n'Replace support.
+     * Depending on the offset in "zoomed" screen pixels
+     * for the vertical direction,
+     * the function returns the steps in unzoomed points
+     * for the autoscroll acceleration
      */
-    void replace( const QPoint &_marker, QString _find, QString _replace, long options );
-
+    double autoScrollAccelerationY( int offset );
 
 public slots:
     void slotScrollVert( int _value );
@@ -278,10 +283,6 @@ public slots:
     void slotMaxColumn( int _max_column );
     void slotMaxRow( int _max_row );
 
-    // Connected to KoFind/KoReplace by KSpreadTable during a search and replace
-    void highlight( const QString &text, int matchingIndex, int matchedLength, const QRect &cellRect );
-    void replace( const QString &newText, int index, int replacedLength,int searchWordLenght, const QRect &cellRect );
-
 protected:
     virtual void keyPressEvent ( QKeyEvent* _ev );
     virtual void paintEvent ( QPaintEvent* _ev );
@@ -292,21 +293,40 @@ protected:
     virtual void wheelEvent( QWheelEvent* );
     virtual void focusInEvent( QFocusEvent* );
     virtual void focusOutEvent( QFocusEvent* );
+    virtual void resizeEvent( QResizeEvent * _ev );
+    virtual void dragMoveEvent(QDragMoveEvent * _ev);
+    virtual void dropEvent(QDropEvent * _ev);
+    virtual void dragLeaveEvent(QDragLeaveEvent * _ev);
+
+private slots:
+    void doAutoScroll();
 
 private:
     virtual void chooseMousePressEvent( QMouseEvent* _ev );
     virtual void chooseMouseReleaseEvent( QMouseEvent* _ev );
     virtual void chooseMouseMoveEvent( QMouseEvent* _ev );
 
-    KSpreadHBorder* hBorderWidget();
-    KSpreadVBorder* vBorderWidget();
-    QScrollBar* horzScrollBar();
-    QScrollBar* vertScrollBar();
-    KSpreadEditWidget* editWidget() { return m_pEditWidget; }
+    virtual bool eventFilter( QObject *o, QEvent *e );
+
+    KSpreadHBorder* hBorderWidget() const;
+    KSpreadVBorder* vBorderWidget() const;
+    QScrollBar* horzScrollBar() const;
+    QScrollBar* vertScrollBar() const;
+    KSpreadEditWidget* editWidget() const { return m_pEditWidget; }
 
     void drawChooseMarker( );
     void drawChooseMarker( const QRect& );
 
+    /**
+     * Clips out the children region from the painter
+     */
+    void clipoutChildren( QPainter& painter, QWMatrix& matrix );
+
+    /**
+     * Paints the children
+     */
+    void paintChildren( QPainter& painter, QWMatrix& matrix );
+
     /**
      * @see #setLastEditorWithFocus
      */
@@ -327,6 +347,7 @@ private:
 
     KSpreadView *m_pView;
     KSpreadDoc* m_pDoc;
+    QTimer * m_scrollTimer;
 
     /**
      * If the user is dragging around with the mouse then this tells us what he is doing.
@@ -345,23 +366,6 @@ private:
     QPoint m_ptGeometryStart;
     QPoint m_ptGeometryEnd;
 
-    /**
-     * The column in which a mouse drag started.
-     */
-    int m_iMouseStartColumn;
-    /**
-     * The row in which a mouse drag started.
-     */
-    int m_iMouseStartRow;
-
-    /**
-     * Tells whether the user selected more than one cell.
-     * If the user presses the left mouse button and if he marks more
-     * than one cell until he releases the button again, then this flag
-     * is set. If this flag is set, then one should repaint all visible
-     * cells once the button is released.
-     */
-    bool m_bMouseMadeSelection;
 
     /**
      * True when the mouse button is pressed
@@ -384,16 +388,27 @@ private:
      */
     QString m_strAnchor;
 
-    float m_fZoom;
+    /**
+     * Non visible range left from current screen
+     * Example:
+     * If the first visible column is 'E', then m_dXOffset stores
+     * the width of the invisible columns 'A' to 'D'.
+     */
+    double m_dXOffset;
 
     /**
-     * Horizontal scroll offset of the view.
+     * Non visible range on top of the current screen
+     * Example:
+     * If the first visible row is '5', then m_dYOffset stores
+     * the height of the invisible rows '1' to '4'.
      */
-    int m_iXOffset;
+    double m_dYOffset;
+
     /**
-     * Vertical scroll offset of the view.
+     * Start coordinates for drag and drop
      */
-    int m_iYOffset;
+    QPoint m_dragStart;
+    bool   m_dragging;
 
     KSpreadLocationEditWidget *m_pPosWidget;
     KSpreadEditWidget *m_pEditWidget;
@@ -416,8 +431,6 @@ private:
      */
     // int m_iMarkerVisible;
 
-    int m_i_chooseMarkerRow;
-    int m_i_chooseMarkerColumn;
 
     /**
      * Is true if the user is to choose a cell.
@@ -431,15 +444,85 @@ private:
      * If a choose selection is started (@ref #startChoose) the current
      * table is saved here.
      */
-    KSpreadTable* m_chooseStartTable;
-
-    // int m_choosePos;
+    KSpreadSheet* m_chooseStartTable;
 
     /**
      * @see #setLastEditorWithFocus
      * @see #lastEditorWithFocus
      */
     EditorType m_focusEditorType;
+
+private:
+
+  void startTheDrag();
+  void paintSelectionChange(QRect area1, QRect area2);
+
+  /**
+   * Small helper function to take a rect representing a selection onscreen and
+   * extend it one cell in every direction (taking into account hidden columns
+   * and rows)
+   */
+  void ExtendRectBorder(QRect& area);
+
+  /* helpers for the paintUpdates function */
+  void paintChooseRect(QPainter& painter, const KoRect &viewRect);
+
+  void paintNormalMarker(QPainter& painter, const KoRect &viewRect);
+
+  void retrieveMarkerInfo( const QRect &marker, const KoRect &viewRect,
+                           double positions[], bool paintSides[] );
+
+
+
+  bool formatKeyPress( QKeyEvent * _ev );
+  double getDouble( KSpreadCell * cell );
+  void convertToDouble( KSpreadCell * cell );
+  void convertToPercent( KSpreadCell * cell );
+  void convertToMoney( KSpreadCell * cell );
+  void convertToTime( KSpreadCell * cell );
+  void convertToDate( KSpreadCell * cell );
+
+  void processClickSelectionHandle(QMouseEvent *event);
+  void processLeftClickAnchor();
+
+
+  /**
+   * Helper function used from many places to extend the current selection such as with
+   * a shift-click, mouse drag, shift-arrow key, etc.
+   *
+   * @param cell coordinates of the cell we want to extend towards.
+   */
+  void extendCurrentSelection(QPoint cell);
+
+  /**
+   * returns the rect that needs to be redrawn
+   */
+  QRect moveDirection(KSpread::MoveTo direction, bool extendSelection);
+
+  void processEnterKey(QKeyEvent *event);
+  void processArrowKey(QKeyEvent *event);
+  void processEscapeKey(QKeyEvent *event);
+  bool processHomeKey(QKeyEvent *event);
+  bool processEndKey(QKeyEvent *event);
+  bool processPriorKey(QKeyEvent *event);
+  bool processNextKey(QKeyEvent *event);
+  void processDeleteKey(QKeyEvent *event);
+  void processF2Key(QKeyEvent *event);
+  void processF4Key(QKeyEvent *event);
+  void processOtherKey(QKeyEvent *event);
+  bool processControlArrowKey(QKeyEvent *event);
+
+  void updateChooseRect(const QPoint &newMarker, const QPoint &newAnchor);
+
+  /**
+   * This function sets the paint dirty flag for a selection change.  The idea
+   * here is that only the edges of the selection need to change (no matter whether
+   * its the real selection or the 'choose' selection).  This calculates which
+   * cells really should look different with the new selection rather than repainting
+   * the entire area
+   */
+  void setSelectionChangePaintDirty(KSpreadSheet* sheet,
+                                    QRect area1, QRect area2);
 };
 
 /**
@@ -449,11 +532,17 @@ class KSpreadHBorder : public QWidget
     Q_OBJECT
 public:
     KSpreadHBorder( QWidget *_parent, KSpreadCanvas *_canvas, KSpreadView *_view  );
+    ~KSpreadHBorder();
 
-    int markerColumn() { return  m_iSelectionAnchor; }
-    void resizeColumn( int resize, int nb = -1,bool makeUndo=true );
-    void adjustColumn( int _col = -1,bool makeUndo=true );
-    void equalizeColumn( int resize);
+    int markerColumn() const { return  m_iSelectionAnchor; }
+    void resizeColumn( double resize, int nb = -1, bool makeUndo = true );
+    void adjustColumn( int _col = -1, bool makeUndo = true );
+    void equalizeColumn( double resize );
+
+    void updateColumns( int from, int to );
+
+private slots:
+    void doAutoScroll();
 
 protected:
     virtual void paintEvent ( QPaintEvent* _ev );
@@ -462,10 +551,14 @@ protected:
     virtual void mouseDoubleClickEvent( QMouseEvent* _ev );
     virtual void mouseMoveEvent( QMouseEvent* _ev );
     virtual void wheelEvent( QWheelEvent* );
+    virtual void focusOutEvent( QFocusEvent* ev );
     void paintSizeIndicator( int mouseX, bool firstTime );
 
+private:
     KSpreadCanvas *m_pCanvas;
     KSpreadView *m_pView;
+    QTimer * m_scrollTimer;
+
     /**
      * Flag that inidicates whether the user wants to mark columns.
      * The user may mark columns by dragging the mouse around in th XBorder widget.
@@ -505,6 +598,13 @@ protected:
      * The label used for showing the current size, when resizing
      */
     QLabel *m_lSize;
+
+    /**
+     * True when the mouse button is pressed
+     */
+    bool m_bMousePressed;
+
+private:
 };
 
 /**
@@ -514,11 +614,16 @@ class KSpreadVBorder : public QWidget
     Q_OBJECT
 public:
     KSpreadVBorder( QWidget *_parent, KSpreadCanvas *_canvas, KSpreadView *_view );
+    ~KSpreadVBorder();
 
-    int markerRow() { return  m_iSelectionAnchor; }
-    void resizeRow( int resize, int nb = -1,bool makeUndo=true );
-    void adjustRow( int _row = -1,bool makeUndo=true );
-    void equalizeRow( int resize);
+    int markerRow() const { return  m_iSelectionAnchor; }
+    void resizeRow( double resize, int nb = -1, bool makeUndo = true );
+    void adjustRow( int _row = -1, bool makeUndo = true );
+    void equalizeRow( double resize );
+    void updateRows( int from, int to );
+
+private slots:
+    void doAutoScroll();
 
 protected:
     virtual void paintEvent ( QPaintEvent* _ev );
@@ -527,10 +632,14 @@ protected:
     virtual void mouseMoveEvent( QMouseEvent* _ev );
     virtual void mouseDoubleClickEvent( QMouseEvent* _ev );
     virtual void wheelEvent( QWheelEvent* );
+    virtual void focusOutEvent( QFocusEvent* ev );
     void paintSizeIndicator( int mouseY, bool firstTime );
 
+private:
     KSpreadCanvas *m_pCanvas;
     KSpreadView *m_pView;
+    QTimer * m_scrollTimer;
+
     bool m_bSelection;
     int m_iSelectionAnchor;
     bool m_bResize;
@@ -540,8 +649,16 @@ protected:
      * The label used for showing the current size, when resizing
      */
     QLabel *m_lSize;
+
+    /**
+     * True when the mouse button is pressed
+     */
+    bool m_bMousePressed;
 };
 
+/*
+ * Tooltip, which displays the comment and cell content, when it's too short
+ */
 class KSpreadToolTip : public QToolTip
 {
 public:
