@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
-   Copyright (C) 1999, 2000, 2001 Montel Laurent <lmontel@mandrakesoft.com>
+   Copyright (C) 1999, 2000, 2001, 2002, 2003 Montel Laurent <lmontel@mandrakesoft.com>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -17,31 +17,27 @@
    Boston, MA 02111-1307, USA.
 */
 
-#include <qprinter.h>
-#include <qgroupbox.h>
-#include <qgrid.h>
 #include <qvbox.h>
-#include <qlabel.h>
 #include <qlayout.h>
 #include <qcheckbox.h>
 #include <qcombobox.h>
+#include <qvgroupbox.h>
 
 #include "kspread_dlg_preference.h"
-#include "kspread_view.h"
-#include "kspread_table.h"
+#include "kspread_sheet.h"
+#include "kspread_sheetprint.h"
 #include "kspread_doc.h"
 #include "kspread_canvas.h"
 #include "kspread_tabbar.h"
 
-#include <kapp.h>
-#include <klocale.h>
-#include <kbuttonbox.h>
-#include <kdialogbase.h>
 #include <kconfig.h>
-#include <kcolorbutton.h>
 #include <kstatusbar.h>
 #include <knuminput.h>
 #include <kspell.h>
+#include <kmessagebox.h>
+#include <kdeversion.h>
+#include <kcolorbutton.h>
+#include <koSpellConfig.h>
 
 KSpreadpreference::KSpreadpreference( KSpreadView* parent, const char* /*name*/)
   : KDialogBase(KDialogBase::IconList,i18n("Configure KSpread") ,
@@ -55,8 +51,8 @@ KSpreadpreference::KSpreadpreference( KSpreadView* parent, const char* /*name*/)
   _preferenceConfig = new  preference(parent,page );
   connect(this, SIGNAL(okClicked()),this,SLOT(slotApply()));
 
-  QVBox *page2=addVBoxPage(i18n("Local Parameters"), QString::null,BarIcon("gohome",KIcon::SizeMedium));
-  parameterLocale *_ParamLocal=new parameterLocale(parent,page2 );
+  QVBox *page2=addVBoxPage(i18n("Locale Parameters"), QString::null,BarIcon("gohome",KIcon::SizeMedium));
+ _localePage=new parameterLocale(parent,page2 );
 
   QVBox *page3=addVBoxPage(i18n("Interface"), QString::null,BarIcon("signature", KIcon::SizeMedium) );
   _configure = new  configure(parent,page3 );
@@ -67,24 +63,45 @@ KSpreadpreference::KSpreadpreference( KSpreadView* parent, const char* /*name*/)
   QVBox *page5=addVBoxPage(i18n("Color"), QString::null,BarIcon("colorize",KIcon::SizeMedium) );
   _colorParameter=new colorParameters(parent,page5 );
 
-  QVBox *page6=addVBoxPage(i18n("Page layout"), QString::null,BarIcon("edit",KIcon::SizeMedium) );
+  QVBox *page6=addVBoxPage(i18n("Page Layout"), QString::null,BarIcon("edit",KIcon::SizeMedium) );
   _layoutPage=new configureLayoutPage(parent,page6 );
 
-  QVBox *page7 = addVBoxPage( i18n("Spelling"), i18n("Spell checker behavior"),
+  QVBox *page7 = addVBoxPage( i18n("Spelling"), i18n("Spell Checker Behavior"),
                           BarIcon("spellcheck", KIcon::SizeMedium) );
   _spellPage=new configureSpellPage(parent,page7);
 
 }
 
+void KSpreadpreference::openPage(int flags)
+{
+    if(flags & KS_PREFERENCES)
+        showPage( 0 );
+    else if(flags & KS_LOCALE)
+        showPage( 1 );
+    else if(flags & KS_INTERFACE)
+        showPage( 2 );
+    else if(flags & KS_MISC)
+        showPage( 3 );
+    else if(flags & KS_COLOR)
+        showPage( 4 );
+    else if(flags & KS_LAYOUT)
+        showPage( 5 );
+    else if(flags & KS_SPELLING)
+        showPage( 6 );
+}
+
 void KSpreadpreference::slotApply()
 {
+  m_pView->doc()->emitBeginOperation( false );
   _preferenceConfig->apply();
   _configure->apply();
   _miscParameter->apply();
   _colorParameter->apply();
   _layoutPage->apply();
   _spellPage->apply();
+  _localePage->apply();
   m_pView->doc()->refreshInterface();
+  m_pView->slotUpdateView( m_pView->activeTable() );
 }
 
 void KSpreadpreference::slotDefault()
@@ -122,39 +139,30 @@ preference::preference( KSpreadView* _view, QVBox *box, char *name )
 
   m_pView = _view;
 
-  QGroupBox* tmpQGroupBox = new QGroupBox( box, "GroupBox" );
-  tmpQGroupBox->setTitle(i18n("Table"));
-  QVBoxLayout *lay1 = new QVBoxLayout(tmpQGroupBox);
-  lay1->addSpacing( 10 );
-  lay1->setMargin( KDialog::marginHint() );
-  lay1->setSpacing( KDialog::spacingHint() );
+  QGroupBox* tmpQGroupBox = new QVGroupBox( i18n("Sheet"), box, "GroupBox" );
 
-  m_pFormula= new QCheckBox(i18n("Show formula"),tmpQGroupBox);
-  lay1->addWidget(m_pFormula);
-  m_pFormula->setChecked(m_pView->activeTable()->getShowFormular());
+  m_pFormula= new QCheckBox(i18n("Show &formula"),tmpQGroupBox);
+  m_pFormula->setChecked(m_pView->activeTable()->getShowFormula());
 
-  m_pGrid=new QCheckBox(i18n("Show Grid"),tmpQGroupBox);
-  lay1->addWidget(m_pGrid);
-  m_pGrid->setChecked(m_pView->activeTable()->getShowGrid());
+  m_pFormulaIndicator= new QCheckBox(i18n("Show &formula indicator"),tmpQGroupBox);
+  m_pFormulaIndicator->setChecked(m_pView->activeTable()->getShowFormulaIndicator());
 
-  m_pColumn=new QCheckBox(i18n("Show column number"),tmpQGroupBox);
-  lay1->addWidget(m_pColumn);
+  m_pGrid=new QCheckBox( i18n("Show &grid"), tmpQGroupBox );
+  m_pGrid->setChecked( m_pView->activeTable()->getShowGrid() );
+
+  m_pColumn=new QCheckBox(i18n("Show c&olumn number"),tmpQGroupBox);
   m_pColumn->setChecked(m_pView->activeTable()->getShowColumnNumber());
 
-  m_pLcMode=new QCheckBox(i18n("LC mode"),tmpQGroupBox);
-  lay1->addWidget(m_pLcMode);
+  m_pLcMode=new QCheckBox(i18n("&LC mode"),tmpQGroupBox);
   m_pLcMode->setChecked(m_pView->activeTable()->getLcMode());
 
-  m_pAutoCalc= new QCheckBox(i18n("Automatic Recalculation"),tmpQGroupBox);
-  lay1->addWidget(m_pAutoCalc);
+  m_pAutoCalc= new QCheckBox(i18n("&Automatic recalculation"),tmpQGroupBox);
   m_pAutoCalc->setChecked(m_pView->activeTable()->getAutoCalc());
 
-  m_pHideZero= new QCheckBox(i18n("Hide Zero"),tmpQGroupBox);
-  lay1->addWidget(m_pHideZero);
+  m_pHideZero= new QCheckBox(i18n("&Hide zero"),tmpQGroupBox);
   m_pHideZero->setChecked(m_pView->activeTable()->getHideZero());
 
-  m_pFirstLetterUpper= new QCheckBox(i18n("Convert first letter to upper case"),tmpQGroupBox);
-  lay1->addWidget(m_pFirstLetterUpper);
+  m_pFirstLetterUpper= new QCheckBox(i18n("Convert first letter to &upper case"),tmpQGroupBox);
   m_pFirstLetterUpper->setChecked(m_pView->activeTable()->getFirstLetterUpper());
 
 }
@@ -163,6 +171,7 @@ preference::preference( KSpreadView* _view, QVBox *box, char *name )
 void preference::slotDefault()
 {
   m_pFormula->setChecked(false);
+  m_pFormulaIndicator->setChecked(true);
   m_pAutoCalc->setChecked(true);
   m_pGrid->setChecked(true);
   m_pColumn->setChecked(false);
@@ -175,7 +184,8 @@ void preference::apply()
 {
   if(m_pView->activeTable()->getLcMode()==m_pLcMode->isChecked()
   && m_pView->activeTable()->getShowColumnNumber()==m_pColumn->isChecked()
-  && m_pView->activeTable()->getShowFormular()==m_pFormula->isChecked()
+  && m_pView->activeTable()->getShowFormula()==m_pFormula->isChecked()
+  && m_pView->activeTable()->getShowFormulaIndicator()==m_pFormulaIndicator->isChecked()
   && m_pView->activeTable()->getAutoCalc()==m_pAutoCalc->isChecked()
   && m_pView->activeTable()->getShowGrid()==m_pGrid->isChecked()
   && m_pView->activeTable()->getHideZero()==m_pHideZero->isChecked()
@@ -185,47 +195,67 @@ void preference::apply()
   }
   else
   {
+        m_pView->doc()->emitBeginOperation( false );
         m_pView->activeTable()->setLcMode(m_pLcMode->isChecked());
         m_pView->activeTable()->setShowColumnNumber(m_pColumn->isChecked());
         m_pView->activeTable()->setShowGrid(m_pGrid->isChecked());
-        m_pView->activeTable()->setShowFormular(m_pFormula->isChecked());
+        m_pView->activeTable()->setShowFormula(m_pFormula->isChecked());
+        m_pView->activeTable()->setShowFormulaIndicator(m_pFormulaIndicator->isChecked());
         m_pView->activeTable()->setAutoCalc(m_pAutoCalc->isChecked());
         m_pView->activeTable()->setHideZero(m_pHideZero->isChecked());
         m_pView->activeTable()->setFirstLetterUpper(m_pFirstLetterUpper->isChecked());
+        m_pView->slotUpdateView( m_pView->activeTable() );
   }
 }
 
 parameterLocale::parameterLocale( KSpreadView* _view, QVBox *box , char *name )
  :QObject ( box->parent(),name)
 {
-  QGroupBox* tmpQGroupBox = new QGroupBox( box, "GroupBox" );
-  tmpQGroupBox->setTitle(i18n("Parameters"));
+    m_pView = _view;
+    m_bUpdateLocale=false;
+  QGroupBox* tmpQGroupBox = new QVGroupBox( i18n("Parameters"), box, "GroupBox" );
 
-  QVBoxLayout *lay1 = new QVBoxLayout(tmpQGroupBox);
-  lay1->addSpacing( 10 );
-  lay1->setMargin( KDialog::marginHint() );
-  lay1->setSpacing( KDialog::spacingHint() );
+  KLocale* locale=_view->doc()->locale();
 
-  QLabel *label=new QLabel( tmpQGroupBox,"label");
-  label->setText( i18n("Language: %1").arg( _view->doc()->locale()->language() ));
-  lay1->addWidget(label);
-  label=new QLabel( tmpQGroupBox,"label6");
-  label->setText( i18n("Number: %1").arg( _view->doc()->locale()->formatNumber(12.55) ));
-  lay1->addWidget(label);
-  label=new QLabel( tmpQGroupBox,"label1");
-  label->setText( i18n("Date: %1").arg( _view->doc()->locale()->formatDate(QDate(2000,10,23)) ));
-  lay1->addWidget(label);
-  label=new QLabel( tmpQGroupBox,"label5");
-  label->setText( i18n("Short date: %1").arg( _view->doc()->locale()->formatDate(QDate(2000,10,23),true) ));
-  lay1->addWidget(label);
-  label=new QLabel( tmpQGroupBox,"label2");
-  label->setText( i18n("Time: %1").arg( _view->doc()->locale()->formatTime(QTime(15,10,53)) ));
-  lay1->addWidget(label);
-  label=new QLabel( tmpQGroupBox,"label3");
-  label->setText( i18n("Money: %1").arg( _view->doc()->locale()->formatMoney(12.55) ));
-  lay1->addWidget(label);
+  m_language=new QLabel( tmpQGroupBox,"label");
+  m_language->setText( i18n("Language: %1").arg( locale->language() ));
+  m_number=new QLabel( tmpQGroupBox,"label6");
+  m_number->setText( i18n("Number: %1").arg( locale->formatNumber(12.55) ));
+  m_date=new QLabel( tmpQGroupBox,"label1");
+  m_date->setText( i18n("Date: %1").arg( locale->formatDate(QDate(2000,10,23)) ));
+  m_shortDate=new QLabel( tmpQGroupBox,"label5");
+  m_shortDate->setText( i18n("Short date: %1").arg( locale->formatDate(QDate(2000,10,23),true) ));
+  m_time=new QLabel( tmpQGroupBox,"label2");
+  m_time->setText( i18n("Time: %1").arg( locale->formatTime(QTime(15,10,53)) ));
+  m_money=new QLabel( tmpQGroupBox,"label3");
+  m_money->setText( i18n("Money: %1").arg( locale->formatMoney(12.55) ));
+
+  m_updateButton=new QPushButton ( i18n("&Update to Locale System"), tmpQGroupBox);
+  connect(m_updateButton, SIGNAL(clicked()),this,SLOT(updateDefaultSystemConfig()));
 }
 
+void parameterLocale::apply()
+{
+    if (m_bUpdateLocale)
+    {
+        m_pView->doc()->emitBeginOperation( false );
+        m_pView->doc()->refreshLocale();
+        m_pView->slotUpdateView( m_pView->activeTable() );
+    }
+}
+
+void parameterLocale::updateDefaultSystemConfig()
+{
+    m_bUpdateLocale=true;
+    static_cast<KSpreadLocale*>(m_pView->doc()->locale())->defaultSystemConfig( );
+    KLocale* locale=m_pView->doc()->locale();
+    m_money->setText( i18n("Money: %1").arg( locale->formatMoney(12.55) ));
+    m_time->setText( i18n("Time: %1").arg( locale->formatTime(QTime(15,10,53)) ));
+    m_shortDate->setText( i18n("Short date: %1").arg( locale->formatDate(QDate(2000,10,23),true) ));
+    m_date->setText( i18n("Date: %1").arg( locale->formatDate(QDate(2000,10,23)) ));
+    m_number->setText( i18n("Number: %1").arg( locale->formatNumber(12.55) ));
+    m_language->setText( i18n("Language: %1").arg( locale->language() ));
+}
 
 configure::configure( KSpreadView* _view, QVBox *box , char *name )
  :QObject ( box->parent(),name)
@@ -239,13 +269,9 @@ configure::configure( KSpreadView* _view, QVBox *box , char *name )
   bool tabbar=true;
   bool formulaBar=true;
   bool statusBar=true;
+  m_oldBackupFile = true;
 
-  QGroupBox* tmpQGroupBox = new QGroupBox( box, "GroupBox" );
-  tmpQGroupBox->setTitle(i18n("Configuration"));
-  QVBoxLayout *lay1 = new QVBoxLayout(tmpQGroupBox);
-  lay1->addSpacing( 10 );
-  lay1->setMargin( KDialog::marginHint() );
-  lay1->setSpacing( KDialog::spacingHint() );
+  QGroupBox* tmpQGroupBox = new QVGroupBox( i18n("Configuration"), box, "GroupBox" );
 
   config = KSpreadFactory::global()->config();
   int _page=1;
@@ -266,51 +292,44 @@ configure::configure( KSpreadView* _view, QVBox *box , char *name )
         statusBar=config->readBoolEntry("Status bar",true);
         oldRecent=config->readNumEntry( "NbRecentFile" ,10);
         oldAutoSaveValue=config->readNumEntry("AutoSave",KoDocument::defaultAutoSave()/60);
+        m_oldBackupFile=config->readBoolEntry("BackupFile",m_oldBackupFile);
         }
   nbPage=new KIntNumInput(_page, tmpQGroupBox , 10);
   nbPage->setRange(1, 10, 1);
-  nbPage->setLabel(i18n("Number of pages open at the beginning:"));
-  lay1->addWidget(nbPage);
+  nbPage->setLabel(i18n("Number of pages open at the &beginning:"));
 
   nbRecentFile=new KIntNumInput(oldRecent, tmpQGroupBox , 10);
   nbRecentFile->setRange(1, 20, 1);
-  nbRecentFile->setLabel(i18n("Number of recent file:"));
-  lay1->addWidget(nbRecentFile);
+  nbRecentFile->setLabel(i18n("&Number of recent files:"));
 
   autoSaveDelay=new KIntNumInput(oldAutoSaveValue, tmpQGroupBox , 10);
   autoSaveDelay->setRange(0, 60, 1);
-  autoSaveDelay->setLabel(i18n("Auto save (min):"));
-  autoSaveDelay->setSpecialValueText(i18n("No auto save"));
+  autoSaveDelay->setLabel(i18n("Au&tosave (min):"));
+  autoSaveDelay->setSpecialValueText(i18n("No autosave"));
   autoSaveDelay->setSuffix(i18n("min"));
-  lay1->addWidget(autoSaveDelay);
 
-  showVScrollBar=new QCheckBox(i18n("Show vertical scrollbar"),tmpQGroupBox);
-  lay1->addWidget(showVScrollBar);
+  m_createBackupFile = new QCheckBox( i18n("Create backup file"), tmpQGroupBox );
+  m_createBackupFile->setChecked( m_oldBackupFile );
+
+  showVScrollBar=new QCheckBox(i18n("Show &vertical scrollbar"),tmpQGroupBox);
   showVScrollBar->setChecked(vertical);
-  showHScrollBar=new QCheckBox(i18n("Show horizontal scrollbar"),tmpQGroupBox);
-  lay1->addWidget(showHScrollBar);
+  showHScrollBar=new QCheckBox(i18n("Show &horizontal scrollbar"),tmpQGroupBox);
   showHScrollBar->setChecked(horizontal);
 
 
-  showColHeader=new QCheckBox(i18n("Show Column Header"),tmpQGroupBox);
-  lay1->addWidget(showColHeader);
+  showColHeader=new QCheckBox(i18n("Show c&olumn header"),tmpQGroupBox);
   showColHeader->setChecked(colHeader);
-  showRowHeader=new QCheckBox(i18n("Show Row Header"),tmpQGroupBox);
-  lay1->addWidget(showRowHeader);
+  showRowHeader=new QCheckBox(i18n("Show &row header"),tmpQGroupBox);
   showRowHeader->setChecked(rowHeader);
 
-  showTabBar =new QCheckBox(i18n("Show tabs"),tmpQGroupBox);
-  lay1->addWidget(showTabBar);
+  showTabBar =new QCheckBox(i18n("Show ta&bs"),tmpQGroupBox);
   showTabBar->setChecked(tabbar);
 
-  showFormulaBar =new QCheckBox(i18n("Show formula toolbar"),tmpQGroupBox);
-  lay1->addWidget(showFormulaBar);
+  showFormulaBar =new QCheckBox(i18n("Sho&w formula toolbar"),tmpQGroupBox);
   showFormulaBar->setChecked(formulaBar);
 
-  showStatusBar =new QCheckBox(i18n("Show statusbar"),tmpQGroupBox);
-  lay1->addWidget(showStatusBar);
+  showStatusBar =new QCheckBox(i18n("Show stat&usbar"),tmpQGroupBox);
   showStatusBar->setChecked(statusBar);
-
 }
 
 
@@ -326,13 +345,16 @@ void configure::slotDefault()
   nbPage->setValue(1);
   nbRecentFile->setValue(10);
   autoSaveDelay->setValue(KoDocument::defaultAutoSave()/60);
+  m_createBackupFile->setChecked( true );
 }
 
 
 void configure::apply()
 {
+    m_pView->doc()->emitBeginOperation( false );
     config->setGroup( "Parameters" );
     config->writeEntry( "NbPage", nbPage->value());
+    KSpreadDoc *doc =m_pView->doc();
     bool active=true;
     active=showHScrollBar->isChecked();
     if( m_pView->horzScrollBar()->isVisible()!=active)
@@ -342,7 +364,7 @@ void configure::apply()
             m_pView->horzScrollBar()->show();
         else
             m_pView->horzScrollBar()->hide();
-        m_pView->doc()->setShowHorizontalScrollBar(active);
+        doc->setShowHorizontalScrollBar(active);
     }
     active=showVScrollBar->isChecked();
     if( m_pView->vertScrollBar()->isVisible()!=active)
@@ -352,7 +374,7 @@ void configure::apply()
             m_pView->vertScrollBar()->show();
         else
             m_pView->vertScrollBar()->hide();
-        m_pView->doc()->setShowVerticalScrollBar(active);
+        doc->setShowVerticalScrollBar(active);
 
     }
     active=showColHeader->isChecked();
@@ -363,7 +385,7 @@ void configure::apply()
             m_pView->hBorderWidget()->show();
         else
             m_pView->hBorderWidget()->hide();
-        m_pView->doc()->setShowColHeader(active);
+        doc->setShowColHeader(active);
     }
 
     active=showRowHeader->isChecked();
@@ -374,7 +396,7 @@ void configure::apply()
             m_pView->vBorderWidget()->show();
         else
             m_pView->vBorderWidget()->hide();
-        m_pView->doc()->setShowRowHeader(active);
+        doc->setShowRowHeader(active);
     }
 
     active=showTabBar->isChecked();
@@ -385,7 +407,7 @@ void configure::apply()
             m_pView->tabBar()->show();
         else
             m_pView->tabBar()->hide();
-        m_pView->doc()->setShowTabBar(active);
+        doc->setShowTabBar(active);
     }
 
     active=showFormulaBar->isChecked();
@@ -397,7 +419,7 @@ void configure::apply()
             m_pView->posWidget()->show();
         else
             m_pView->posWidget()->hide();
-        m_pView->doc()->setShowFormularBar(active);
+        doc->setShowFormulaBar(active);
     }
     active=showStatusBar->isChecked();
     if(m_pView->statusBar() && m_pView->statusBar()->isVisible()!=active)
@@ -407,7 +429,7 @@ void configure::apply()
             m_pView->statusBar()->show();
         else
             m_pView->statusBar()->hide();
-        m_pView->doc()->setShowStatusBar(active);
+        doc->setShowStatusBar(active);
     }
     int val=nbRecentFile->value();
     if( oldRecent!= val)
@@ -419,8 +441,18 @@ void configure::apply()
     if(val!=oldAutoSaveValue)
     {
         config->writeEntry( "AutoSave", val );
-        m_pView->doc()->setAutoSave(val*60);
+        doc->setAutoSave(val*60);
     }
+
+    bool state =m_createBackupFile->isChecked();
+    if(state!=m_oldBackupFile)
+    {
+        config->writeEntry( "BackupFile", state );
+        doc->setBackupFile( state);
+        m_oldBackupFile=state;
+    }
+
+    m_pView->slotUpdateView( m_pView->activeTable() );
 }
 
 
@@ -430,30 +462,24 @@ miscParameters::miscParameters( KSpreadView* _view,QVBox *box, char *name )
   m_pView = _view;
 
 
-  QGroupBox* tmpQGroupBox = new QGroupBox( box, "GroupBox" );
-  tmpQGroupBox->setTitle(i18n("Misc"));
-  QVBoxLayout *lay1 = new QVBoxLayout(tmpQGroupBox);
-  lay1->addSpacing( 10 );
-  lay1->setMargin( KDialog::marginHint() );
-  lay1->setSpacing( KDialog::spacingHint() );
+  QGroupBox* tmpQGroupBox = new QVGroupBox( i18n("Misc"), box, "GroupBox" );
 
   config = KSpreadFactory::global()->config();
-  int _indent=10;
+  double _indent = 10.0;
   bool m_bMsgError=false;
   bool m_bCommentIndicator=true;
   if( config->hasGroup("Parameters" ))
         {
         config->setGroup( "Parameters" );
-        _indent=config->readNumEntry( "Indent" ,10) ;
+        _indent = config->readNumEntry( "Indent" , 10.0 ) ;
         m_bMsgError=config->readBoolEntry( "Msg error" ,false) ;
 	m_bCommentIndicator=config->readBoolEntry( "Comment Indicator",true);
         }
 
-  QLabel *label=new QLabel(tmpQGroupBox);
-  label->setText(i18n("Completion mode:"));
-  lay1->addWidget(label);
+  QLabel *label=new QLabel(i18n("&Completion mode:"), tmpQGroupBox);
 
-  typeCompletion=new QComboBox( tmpQGroupBox);
+  typeCompletion=new QComboBox(tmpQGroupBox);
+  label->setBuddy(typeCompletion);
   QStringList listType;
   listType+=i18n("None");
   listType+=i18n("Manual");
@@ -462,48 +488,47 @@ miscParameters::miscParameters( KSpreadView* _view,QVBox *box, char *name )
   listType+=i18n("Semi-Automatic");
   typeCompletion->insertStringList(listType);
   typeCompletion->setCurrentItem(0);
-  lay1->addWidget(typeCompletion);
   comboChanged=false;
   connect(typeCompletion,SIGNAL(activated( const QString & )),this,SLOT(slotTextComboChanged(const QString &)));
 
-  valIndent=new KIntNumInput(_indent, tmpQGroupBox , 10);
-  valIndent->setRange(1, 100, 1);
-  valIndent->setLabel(i18n("Value of indent:"));
-  lay1->addWidget(valIndent);
+//   valIndent = new KDoubleNumInput( _indent, tmpQGroupBox , 10.0 );
+  valIndent = new KDoubleNumInput( tmpQGroupBox );
+  valIndent->setRange( KoUnit::ptToUnit( 0.0, _view->doc()->getUnit() ),
+                       KoUnit::ptToUnit( 400.0, _view->doc()->getUnit() ),
+                       KoUnit::ptToUnit( 10.0, _view->doc()->getUnit()) );
+  valIndent->setRange( 0.0, 100.0, 10.0 );
+  valIndent->setValue ( KoUnit::ptToUnit( _indent, _view->doc()->getUnit() ) );
+  valIndent->setLabel(i18n("&Value of indent:"));
 
-  label=new QLabel(tmpQGroupBox);
-  label->setText(i18n("Press enter to move selection to:"));
-  lay1->addWidget(label);
+  label=new QLabel(i18n("&Press enter to move selection to:"), tmpQGroupBox);
   typeOfMove=new QComboBox( tmpQGroupBox);
+  label->setBuddy(typeOfMove);
   listType.clear();
-  listType+=i18n("Bottom");
-  listType+=i18n("Top");
-  listType+=i18n("Right");
-  listType+=i18n("Left");
+  listType+=i18n("towards to", "Bottom");
+  listType+=i18n("towards to", "Top");
+  listType+=i18n("towards to", "Right");
+  listType+=i18n("towards to", "Left");
+  listType+=i18n("towards to", "Bottom, First Cell");
   typeOfMove->insertStringList(listType);
   typeOfMove->setCurrentItem(0);
-  lay1->addWidget(typeOfMove);
-  msgError= new QCheckBox(i18n("Show error message"),tmpQGroupBox);
+  msgError= new QCheckBox(i18n("&Show error message"),tmpQGroupBox);
   msgError->setChecked(m_bMsgError);
-  lay1->addWidget(msgError);
 
-  label=new QLabel(tmpQGroupBox);
-  label->setText(i18n("Method of calc:"));
-  lay1->addWidget(label);
+  label=new QLabel(i18n("&Method of calc:"), tmpQGroupBox);
 
   typeCalc=new QComboBox( tmpQGroupBox);
+  label->setBuddy(typeCalc);
   QStringList listTypeCalc;
   listTypeCalc+=i18n("Sum");
   listTypeCalc+=i18n("Min");
   listTypeCalc+=i18n("Max");
   listTypeCalc+=i18n("Average");
   listTypeCalc+=i18n("Count");
+  listTypeCalc+=i18n("None");
   typeCalc->insertStringList(listTypeCalc);
   typeCalc->setCurrentItem(0);
-  lay1->addWidget(typeCalc);
-  commentIndicator=new QCheckBox(i18n("Show comment indicator"),tmpQGroupBox);
+  commentIndicator=new QCheckBox(i18n("Show comment &indicator"),tmpQGroupBox);
   commentIndicator->setChecked(m_bCommentIndicator);
-  lay1->addWidget(commentIndicator);
 
   initComboBox();
 }
@@ -545,18 +570,21 @@ switch(tmpCompletion )
         }
 switch( m_pView->doc()->getMoveToValue( ))
         {
-        case  Bottom:
+        case  KSpread::Bottom:
                 typeOfMove->setCurrentItem(0);
                 break;
-        case  Left:
+        case  KSpread::Left:
                 typeOfMove->setCurrentItem(3);
                 break;
-        case  Top:
+        case  KSpread::Top:
                 typeOfMove->setCurrentItem(1);
                 break;
-        case  Right:
+        case  KSpread::Right:
                 typeOfMove->setCurrentItem(2);
                 break;
+        case  KSpread::BottomFirst:
+                typeOfMove->setCurrentItem(4);
+                break;
         default :
                 typeOfMove->setCurrentItem(0);
                 break;
@@ -579,6 +607,9 @@ switch( m_pView->doc()->getTypeOfCalc())
         case  Count:
 	        typeCalc->setCurrentItem(4);
                 break;
+        case  NoneCalc:
+	        typeCalc->setCurrentItem(5);
+                break;
         default :
                 typeCalc->setCurrentItem(0);
                 break;
@@ -588,7 +619,7 @@ switch( m_pView->doc()->getTypeOfCalc())
 
 void miscParameters::slotDefault()
 {
-  valIndent->setValue(10);
+  valIndent->setValue( 10.0 );
   typeCompletion->setCurrentItem(3);
   typeOfMove->setCurrentItem(0);
   msgError->setChecked(false);
@@ -628,20 +659,23 @@ void miscParameters::apply()
         config->writeEntry( "Completion Mode", (int)tmpCompletion);
     }
 
-    MoveTo tmpMoveTo=Bottom;
+    KSpread::MoveTo tmpMoveTo=KSpread::Bottom;
     switch(typeOfMove->currentItem())
     {
         case 0:
-            tmpMoveTo=Bottom;
+            tmpMoveTo=KSpread::Bottom;
             break;
         case 1:
-            tmpMoveTo=Top;
+            tmpMoveTo=KSpread::Top;
             break;
         case 2:
-            tmpMoveTo=Right;
+            tmpMoveTo=KSpread::Right;
             break;
         case 3:
-            tmpMoveTo=Left;
+            tmpMoveTo=KSpread::Left;
+            break;
+        case 4:
+            tmpMoveTo=KSpread::BottomFirst;
             break;
     }
     if(tmpMoveTo!=m_pView->doc()->getMoveToValue())
@@ -668,6 +702,10 @@ void miscParameters::apply()
 	case 4:
             tmpMethodCalc=Count;
             break;
+        case 5:
+            tmpMethodCalc=NoneCalc;
+            break;
+
     }
     if(tmpMethodCalc!=m_pView->doc()->getTypeOfCalc())
     {
@@ -677,11 +715,11 @@ void miscParameters::apply()
         m_pView->initCalcMenu();
     }
 
-    int val=valIndent->value();
-    if(val!=m_pView->doc()->getIndentValue())
+    double val = valIndent->value();
+    if( val != m_pView->doc()->getIndentValue() )
     {
-        m_pView->doc()->setIndentValue( val);
-        config->writeEntry( "Indent", val);
+        m_pView->doc()->setIndentValue( val );
+        config->writeEntry( "Indent", val );
     }
 
     bool active=msgError->isChecked();
@@ -709,39 +747,62 @@ colorParameters::colorParameters( KSpreadView* _view,QVBox *box , char *name )
 
   QColor _gridColor(Qt::lightGray);
 
-if(  config->hasGroup("KSpread Color" ) )
-   {
-     config->setGroup( "KSpread Color" );
-     _gridColor= config->readColorEntry("GridColor",&_gridColor);
-   }
+  if ( config->hasGroup("KSpread Color" ) )
+  {
+    config->setGroup( "KSpread Color" );
+    _gridColor = config->readColorEntry("GridColor",&_gridColor);
+  }
 
-  QGroupBox* tmpQGroupBox = new QGroupBox( box, "GroupBox" );
-  tmpQGroupBox->setTitle(i18n("Color"));
-  QGridLayout *grid1 = new QGridLayout(tmpQGroupBox,5,1,15,7);
-  QLabel *lab=new QLabel( tmpQGroupBox,"label20");
-  lab->setText( i18n("Grid Color:"));
-  grid1->addWidget(lab,0,0);
+  QGroupBox* tmpQGroupBox = new QVGroupBox( i18n("Color"), box, "GroupBox" );
 
-  gridColor=new KColorButton(tmpQGroupBox);
-  gridColor->setColor(_gridColor);
-  grid1->addWidget(gridColor,1,0);
+  QLabel *label = new QLabel(i18n("&Grid color:"), tmpQGroupBox,"label20" );
 
+  gridColor = new KColorButton( _gridColor,
+                                Qt::lightGray,
+                                tmpQGroupBox );
+
+  label->setBuddy(gridColor);
+
+  QColor _pbColor(Qt::red);
+  if ( config->hasGroup("KSpread Color" ) )
+  {
+    config->setGroup( "KSpread Color" );
+    _pbColor = config->readColorEntry("PageBorderColor", &_pbColor);
+  }
+
+  QLabel * label2 = new QLabel( i18n("&Page borders:"), tmpQGroupBox, "label21" );
+
+  pageBorderColor = new KColorButton( _pbColor,
+                                Qt::red,
+                                tmpQGroupBox );
+
+
+  label2->setBuddy(pageBorderColor);
 }
 
 void colorParameters::apply()
 {
-    QColor _col=gridColor->color();
-    if(m_pView->doc()->defaultGridPen().color()!=_col)
-        {
-	 m_pView->doc()->changeDefaultGridPenColor( _col);
-	 config->setGroup( "KSpread Color" );
-	 config->writeEntry("GridColor",_col);
-	}
+  QColor _col = gridColor->color();
+  if ( m_pView->doc()->defaultGridPen().color() != _col )
+  {
+    m_pView->doc()->changeDefaultGridPenColor( _col );
+    config->setGroup( "KSpread Color" );
+    config->writeEntry( "GridColor", _col );
+  }
+
+  QColor _pbColor = pageBorderColor->color();
+  if ( m_pView->doc()->pageBorderColor() != _pbColor )
+  {
+    m_pView->doc()->changePageBorderColor( _pbColor );
+    config->setGroup( "KSpread Color" );
+    config->writeEntry( "PageBorderColor", _pbColor );
+  }
 }
 
 void colorParameters::slotDefault()
 {
-    gridColor->setColor(lightGray);
+  gridColor->setColor( lightGray );
+  pageBorderColor->setColor( red );
 }
 
 
@@ -751,31 +812,31 @@ configureLayoutPage::configureLayoutPage( KSpreadView* _view,QVBox *box , char *
 {
   m_pView = _view;
 
-  QGroupBox* tmpQGroupBox = new QGroupBox( box, "GroupBox" );
-  tmpQGroupBox->setTitle(i18n("Default parameters"));
+  QGroupBox* tmpQGroupBox = new QGroupBox( 0, Qt::Vertical, i18n("Default Parameters"), box, "GroupBox" );
+  tmpQGroupBox->layout()->setSpacing(KDialog::spacingHint());
+  tmpQGroupBox->layout()->setMargin(KDialog::marginHint());
 
-
-  QGridLayout *grid1 = new QGridLayout(tmpQGroupBox,8,1, KDialog::marginHint()+10, KDialog::spacingHint());
+  QGridLayout *grid1 = new QGridLayout(tmpQGroupBox->layout(),8,1);
   grid1->addRowSpacing( 0, KDialog::marginHint()  );
   grid1->setRowStretch( 7, 10 );
 
   config = KSpreadFactory::global()->config();
 
-  QLabel *label=new QLabel(tmpQGroupBox);
-  label->setText(i18n("Default page size:"));
+  QLabel *label=new QLabel(i18n("Default page &size:"), tmpQGroupBox);
 
   grid1->addWidget(label,0,0);
 
   defaultSizePage=new QComboBox( tmpQGroupBox);
+  label->setBuddy(defaultSizePage);
   defaultSizePage->insertStringList( KoPageFormat::allFormats() );
   defaultSizePage->setCurrentItem(1);
   grid1->addWidget(defaultSizePage,1,0);
 
-  label=new QLabel(tmpQGroupBox);
-  label->setText(i18n("Default page orientation:"));
+  label=new QLabel(i18n("Default page &orientation:"), tmpQGroupBox);
   grid1->addWidget(label,2,0);
 
   defaultOrientationPage=new QComboBox( tmpQGroupBox);
+  label->setBuddy(defaultOrientationPage);
   QStringList listType;
   listType+=i18n( "Portrait" );
   listType+=i18n( "Landscape" );
@@ -784,13 +845,20 @@ configureLayoutPage::configureLayoutPage( KSpreadView* _view,QVBox *box , char *
   grid1->addWidget(defaultOrientationPage,3,0);
 
   label=new QLabel(tmpQGroupBox);
-  label->setText(i18n("Default page units:"));
+  label->setText(i18n("Default page &units:"));
   grid1->addWidget(label,4,0);
   defaultUnit=new QComboBox( tmpQGroupBox);
+  label->setBuddy(defaultUnit);
   listType.clear();
-  listType=i18n( "Millimeters (mm)" ) ;
-  listType+=i18n( "Points (pt)" ) ;
-  listType+=i18n( "Inches (in)" );
+  listType=KoUnit::unitDescription( KoUnit::U_MM );
+  listType+=KoUnit::unitDescription( KoUnit::U_PT );
+  listType+=KoUnit::unitDescription( KoUnit::U_INCH );
+  listType+=KoUnit::unitDescription( KoUnit::U_CM );
+  listType+=KoUnit::unitDescription( KoUnit::U_PI );
+  listType+=KoUnit::unitDescription( KoUnit::U_CC );
+  listType+=KoUnit::unitDescription( KoUnit::U_DD );
+  listType+=KoUnit::unitDescription( KoUnit::U_DM );
+
   defaultUnit->insertStringList(listType);
   defaultUnit->setCurrentItem(0);
   grid1->addWidget(defaultUnit,5,0);
@@ -807,44 +875,79 @@ void configureLayoutPage::slotDefault()
 
 void configureLayoutPage::initCombo()
 {
-  paper=1;
-  orientation=0;
-  unit=0;
-  if( config->hasGroup("KSpread Page Layout" ))
+    paper=1;
+    orientation=0;
+    unit=0;
+    if( config->hasGroup("KSpread Page Layout" ))
     {
-      config->setGroup( "KSpread Page Layout" );
-      paper=config->readNumEntry( "Default size page" ,1);
-      orientation=config->readNumEntry( "Default orientation page" ,0);
-      unit=config->readNumEntry( "Default unit page" ,0);
+        config->setGroup( "KSpread Page Layout" );
+        paper=config->readNumEntry( "Default size page" ,1);
+        orientation=config->readNumEntry( "Default orientation page" ,0);
+        unit=config->readNumEntry( "Default unit page" ,0);
     }
-  defaultSizePage->setCurrentItem(paper);
-  defaultOrientationPage->setCurrentItem(orientation);
-  defaultUnit->setCurrentItem(unit);
+
+    switch (m_pView->doc()->getUnit() )
+    {
+    case KoUnit::U_MM:
+        unit=0;
+        break;
+    case KoUnit::U_PT:
+        unit=1;
+        break;
+    case KoUnit::U_INCH:
+        unit=2;
+        break;
+    case KoUnit::U_CM:
+        unit=3;
+        break;
+    case KoUnit::U_DM:
+        unit = 4;
+        break;
+    case KoUnit::U_PI:
+        unit = 5;
+        break;
+    case KoUnit::U_DD:
+        unit = 6;
+        break;
+    case KoUnit::U_CC:
+        unit = 7;
+        break;
+    default:
+        unit=3;
+    }
+    defaultUnit->setCurrentItem(unit);
+
+
+    defaultSizePage->setCurrentItem(paper);
+    defaultOrientationPage->setCurrentItem(orientation);
+    defaultUnit->setCurrentItem(unit);
 }
 
 
 void configureLayoutPage::apply()
 {
+  m_pView->doc()->emitBeginOperation( false );
   config->setGroup( "KSpread Page Layout" );
 
- if(paper!=defaultSizePage->currentItem())
-   {
-     unsigned int sizePage=defaultSizePage->currentItem();
-     config->writeEntry( "Default size page", sizePage);
-     m_pView->doc()->setPaperFormat((KoFormat)sizePage);
-   }
- if(orientation!=defaultOrientationPage->currentItem())
-   {
-     unsigned int orientationPage=defaultOrientationPage->currentItem();
-     config->writeEntry( "Default orientation page", orientationPage);
-     m_pView->doc()->setPaperOrientation((KoOrientation)orientationPage);
-   }
- if(unit!=defaultUnit->currentItem())
-   {
-     unsigned int unitPage=defaultUnit->currentItem();
-     config->writeEntry( "Default unit page", unitPage);
-     m_pView->doc()->setPaperUnit((KoUnit)unitPage);
-   }
+  if( paper != defaultSizePage->currentItem() )
+  {
+     unsigned int sizePage = defaultSizePage->currentItem();
+     config->writeEntry( "Default size page", sizePage );
+     m_pView->activeTable()->print()->setPaperFormat( (KoFormat)sizePage );
+  }
+  if( orientation != defaultOrientationPage->currentItem() )
+  {
+     unsigned int orientationPage = defaultOrientationPage->currentItem();
+     config->writeEntry( "Default orientation page", orientationPage );
+     m_pView->activeTable()->print()->setPaperOrientation( (KoOrientation)orientationPage );
+  }
+  if( unit != defaultUnit->currentItem() )
+  {
+     unsigned int unitPage = defaultUnit->currentItem();
+     config->writeEntry( "Default unit page", unitPage );
+     m_pView->doc()->setUnit( (KoUnit::Unit)unitPage );
+  }
+  m_pView->slotUpdateView( m_pView->activeTable() );
 }
 
 configureSpellPage::configureSpellPage( KSpreadView* _view,QVBox *box , char *name )
@@ -853,18 +956,28 @@ configureSpellPage::configureSpellPage( KSpreadView* _view,QVBox *box , char *na
   m_pView = _view;
 
   config = KSpreadFactory::global()->config();
-  QGroupBox* tmpQGroupBox = new QGroupBox( box, "GroupBox" );
-  tmpQGroupBox->setTitle(i18n("Spelling"));
-  QGridLayout *grid1 = new QGridLayout(tmpQGroupBox,8,1, KDialog::marginHint()+10, KDialog::spacingHint());
-  grid1->addRowSpacing( 0, KDialog::marginHint() + 5 );
-  grid1->setRowStretch( 7, 10 );
 
-  _spellConfig  = new KSpellConfig(tmpQGroupBox, 0L ,m_pView->doc()->getKSpellConfig(), false );
-  grid1->addWidget(_spellConfig,0,0);
+
+  m_spellConfigWidget = new KSpellConfig( box, "spell_check",m_pView->doc()->getKSpellConfig()/*, false*/);
+    dontCheckUpperWord = new QCheckBox( i18n("Do not check upper word"),box);
+    dontCheckTitleCase = new QCheckBox( i18n("Do not check title case"),box);
+
+    if( config->hasGroup("KSpell kspread") )
+    {
+        config->setGroup( "KSpell kspread" );
+
+        dontCheckUpperWord->setChecked(config->readBoolEntry("KSpell_dont_check_upper_word",false));
+        dontCheckTitleCase->setChecked(config->readBoolEntry("KSpell_dont_check_title_case",false));
+    }
+    //m_spellConfigWidget->addIgnoreList( m_pView->doc()->spellListIgnoreAll() );
 }
 
+
 void configureSpellPage::apply()
 {
+  m_pView->doc()->emitBeginOperation( false );
+
+  KSpellConfig *_spellConfig = m_spellConfigWidget;
   config->setGroup( "KSpell kspread" );
   config->writeEntry ("KSpell_NoRootAffix",(int) _spellConfig->noRootAffix ());
   config->writeEntry ("KSpell_RunTogether", (int) _spellConfig->runTogether ());
@@ -872,17 +985,27 @@ void configureSpellPage::apply()
   config->writeEntry ("KSpell_DictFromList",(int)  _spellConfig->dictFromList());
   config->writeEntry ("KSpell_Encoding", (int)  _spellConfig->encoding());
   config->writeEntry ("KSpell_Client",  _spellConfig->client());
-  m_pView->doc()->setKSpellConfig(*_spellConfig);
+//  m_spellConfigWidget->saveDictionary();
+  KSpreadDoc* doc = m_pView->doc();
+  doc->setKSpellConfig(*_spellConfig);
+
+    bool state=dontCheckUpperWord->isChecked();
+  config->writeEntry ("KSpell_dont_check_upper_word",(int)state);
+  doc->setDontCheckUpperWord(state);
+
+  state=dontCheckTitleCase->isChecked();
+  config->writeEntry("KSpell_dont_check_title_case",(int)state);
+  doc->setDontCheckTitleCase(state);
+
+  //m_pView->doc()->addIgnoreWordAllList( m_spellConfigWidget->ignoreList() );
+
+  m_pView->slotUpdateView( m_pView->activeTable() );
 }
 
 void configureSpellPage::slotDefault()
 {
-    _spellConfig->setNoRootAffix( 0);
-    _spellConfig->setRunTogether(0);
-    _spellConfig->setDictionary( "");
-    _spellConfig->setDictFromList( FALSE);
-    _spellConfig->setEncoding (KS_E_ASCII);
-    _spellConfig->setClient (KS_CLIENT_ISPELL);
+    //FIXME
+    //m_spellConfigWidget->setDefault();
 }
 
 #include "kspread_dlg_preference.moc"
