@@ -18,23 +18,82 @@
    Boston, MA 02111-1307, USA.
 */
 
-#include <iostream>
-
 #include <qpainter.h>
 
+#include <kdebug.h>
+#include <klocale.h>
+
+#include "elementvisitor.h"
 #include "indexelement.h"
 #include "formulacursor.h"
 #include "formulaelement.h"
+#include "kformulacommand.h"
 #include "sequenceelement.h"
 
+
 KFORMULA_NAMESPACE_BEGIN
-using namespace std;
+
+
+class IndexSequenceElement : public SequenceElement {
+    typedef SequenceElement inherited;
+public:
+
+    IndexSequenceElement( BasicElement* parent = 0 ) : SequenceElement( parent ) {}
+    virtual IndexSequenceElement* clone() {
+        return new IndexSequenceElement( *this );
+    }
+
+    /**
+     * This is called by the container to get a command depending on
+     * the current cursor position (this is how the element gets chosen)
+     * and the request.
+     *
+     * @returns the command that performs the requested action with
+     * the containers active cursor.
+     */
+    virtual KCommand* buildCommand( Container*, Request* );
+};
+
+
+KCommand* IndexSequenceElement::buildCommand( Container* container, Request* request )
+{
+    FormulaCursor* cursor = container->activeCursor();
+    if ( cursor->isReadOnly() ) {
+        return 0;
+    }
+
+    switch ( *request ) {
+    case req_addIndex: {
+        FormulaCursor* cursor = container->activeCursor();
+        if ( cursor->isSelection() ||
+             ( cursor->getPos() > 0 && cursor->getPos() < countChildren() ) ) {
+            break;
+        }
+        IndexElement* element = static_cast<IndexElement*>( getParent() );
+        IndexRequest* ir = static_cast<IndexRequest*>( request );
+        ElementIndexPtr index = element->getIndex( ir->index() );
+        if ( !index->hasIndex() ) {
+            KFCAddGenericIndex* command = new KFCAddGenericIndex( container, index );
+            return command;
+        }
+        else {
+            index->moveToIndex( cursor, afterCursor );
+            cursor->setSelection( false );
+            formula()->cursorHasMoved( cursor );
+            return 0;
+        }
+    }
+    default:
+        break;
+    }
+    return inherited::buildCommand( container, request );
+}
 
 
 IndexElement::IndexElement(BasicElement* parent)
     : BasicElement(parent)
 {
-    content = new SequenceElement(this);
+    content = new IndexSequenceElement( this );
 
     upperLeft   = 0;
     upperMiddle = 0;
@@ -56,15 +115,111 @@ IndexElement::~IndexElement()
 }
 
 
+IndexElement::IndexElement( const IndexElement& other )
+    : BasicElement( other )
+{
+    content = new IndexSequenceElement( *dynamic_cast<IndexSequenceElement*>( other.content ) );
+
+    if ( other.upperLeft ) {
+        upperLeft = new SequenceElement( *( other.upperLeft ) );
+        upperLeft->setParent( this );
+    }
+    else {
+        upperLeft = 0;
+    }
+    if ( other.upperMiddle ) {
+        upperMiddle = new SequenceElement( *( other.upperMiddle ) );
+        upperMiddle->setParent( this );
+    }
+    else {
+        upperMiddle = 0;
+    }
+    if ( other.upperRight ) {
+        upperRight = new SequenceElement( *( other.upperRight ) );
+        upperRight->setParent( this );
+    }
+    else {
+        upperRight = 0;
+    }
+
+    if ( other.lowerLeft ) {
+        lowerLeft = new SequenceElement( *( other.lowerLeft ) );
+        lowerLeft->setParent( this );
+    }
+    else {
+        lowerLeft = 0;
+    }
+    if ( other.lowerMiddle ) {
+        lowerMiddle = new SequenceElement( *( other.lowerMiddle ) );
+        lowerMiddle->setParent( this );
+    }
+    else {
+        lowerMiddle = 0;
+    }
+    if ( other.lowerRight ) {
+        lowerRight = new SequenceElement( *( other.lowerRight ) );
+        lowerRight->setParent( this );
+    }
+    else {
+        lowerRight = 0;
+    }
+}
+
+
+bool IndexElement::accept( ElementVisitor* visitor )
+{
+    return visitor->visit( this );
+}
+
+
+QChar IndexElement::getCharacter() const
+{
+    if ( !content->isTextOnly() ) {
+        return QChar::null;
+    }
+
+    if ( hasUpperRight() && !upperRight->isTextOnly() ) {
+        return QChar::null;
+    }
+    if ( hasUpperMiddle() && !upperMiddle->isTextOnly() ) {
+        return QChar::null;
+    }
+    if ( hasUpperLeft() && !upperLeft->isTextOnly() ) {
+        return QChar::null;
+    }
+    if ( hasLowerRight() && !lowerRight->isTextOnly() ) {
+        return QChar::null;
+    }
+    if ( hasLowerMiddle() && !lowerMiddle->isTextOnly() ) {
+        return QChar::null;
+    }
+    if ( hasLowerLeft() && !lowerLeft->isTextOnly() ) {
+        return QChar::null;
+    }
+
+    return ' ';
+}
+
+void IndexElement::entered( SequenceElement* child )
+{
+    if ( child == content ) {
+        formula()->tell( i18n( "Indexed list" ) );
+    }
+    else {
+        formula()->tell( i18n( "Index" ) );
+    }
+}
+
+
 /**
  * Returns the element the point is in.
  */
-BasicElement* IndexElement::goToPos(FormulaCursor* cursor, bool& handled,
-                                    const KoPoint& point, const KoPoint& parentOrigin)
+BasicElement* IndexElement::goToPos( FormulaCursor* cursor, bool& handled,
+                                     const LuPixelPoint& point, const LuPixelPoint& parentOrigin )
 {
     BasicElement* e = BasicElement::goToPos(cursor, handled, point, parentOrigin);
     if (e != 0) {
-        KoPoint myPos(parentOrigin.x()+getX(), parentOrigin.y()+getY());
+        LuPixelPoint myPos(parentOrigin.x()+getX(), parentOrigin.y()+getY());
         e = content->goToPos(cursor, handled, point, myPos);
         if (e != 0) return e;
 
@@ -93,8 +248,8 @@ BasicElement* IndexElement::goToPos(FormulaCursor* cursor, bool& handled,
             if (e != 0) return e;
         }
 
-        double dx = point.x() - myPos.x();
-        double dy = point.y() - myPos.y();
+        luPixel dx = point.x() - myPos.x();
+        luPixel dy = point.y() - myPos.y();
 
         // the positions after the left indexes
         if (dx < content->getX()+content->getWidth()) {
@@ -176,77 +331,68 @@ void IndexElement::setMiddleX(int xOffset, int middleWidth)
  * Calculates our width and height and
  * our children's parentPosition.
  */
-void IndexElement::calcSizes(const ContextStyle& contextStyle, ContextStyle::TextStyle tstyle, ContextStyle::IndexStyle istyle)
+void IndexElement::calcSizes(const ContextStyle& style, ContextStyle::TextStyle tstyle, ContextStyle::IndexStyle istyle)
 {
-    //int distX = contextStyle.getDistanceX(tstyle);
-    double distY = contextStyle.getThinSpace( tstyle );
+    luPixel distY = style.ptToPixelY( style.getThinSpace( tstyle ) );
+
+    ContextStyle::TextStyle i_tstyle = style.convertTextStyleIndex(tstyle);
+    ContextStyle::IndexStyle u_istyle = style.convertIndexStyleUpper( istyle );
+    ContextStyle::IndexStyle l_istyle = style.convertIndexStyleLower( istyle );
 
     // get the indexes size
-    double ulWidth = 0, ulHeight = 0, ulMidline = 0;
+    luPixel ulWidth = 0, ulHeight = 0, ulMidline = 0;
     if (hasUpperLeft()) {
-        upperLeft->calcSizes(contextStyle,
-			     contextStyle.convertTextStyleIndex(tstyle),
-			     contextStyle.convertIndexStyleUpper(istyle));
+        upperLeft->calcSizes( style, i_tstyle, u_istyle );
         ulWidth = upperLeft->getWidth();
         ulHeight = upperLeft->getHeight();
-        ulMidline = upperLeft->getMidline();
+        ulMidline = upperLeft->axis( style, i_tstyle );
     }
 
-    double umWidth = 0, umHeight = 0, umMidline = 0;
+    luPixel umWidth = 0, umHeight = 0, umMidline = 0;
     if (hasUpperMiddle()) {
-	upperMiddle->calcSizes(contextStyle,
-			       contextStyle.convertTextStyleIndex(tstyle),
-			       contextStyle.convertIndexStyleUpper(istyle));
+	upperMiddle->calcSizes( style, i_tstyle, u_istyle );
         umWidth = upperMiddle->getWidth();
         umHeight = upperMiddle->getHeight() + distY;
-        umMidline = upperMiddle->getMidline();
+        umMidline = upperMiddle->axis( style, i_tstyle );
     }
 
-    double urWidth = 0, urHeight = 0, urMidline = 0;
+    luPixel urWidth = 0, urHeight = 0, urMidline = 0;
     if (hasUpperRight()) {
-        upperRight->calcSizes(contextStyle,
-			      contextStyle.convertTextStyleIndex(tstyle),
-			      contextStyle.convertIndexStyleUpper(istyle));
+        upperRight->calcSizes( style, i_tstyle, u_istyle );
         urWidth = upperRight->getWidth();
         urHeight = upperRight->getHeight();
-        urMidline = upperRight->getMidline();
+        urMidline = upperRight->axis( style, i_tstyle );
     }
 
-    double llWidth = 0, llHeight = 0, llMidline = 0;
+    luPixel llWidth = 0, llHeight = 0, llMidline = 0;
     if (hasLowerLeft()) {
-        lowerLeft->calcSizes(contextStyle,
-			     contextStyle.convertTextStyleIndex(tstyle),
-			     contextStyle.convertIndexStyleLower(istyle));
+        lowerLeft->calcSizes( style, i_tstyle, l_istyle );
         llWidth = lowerLeft->getWidth();
         llHeight = lowerLeft->getHeight();
-        llMidline = lowerLeft->getMidline();
+        llMidline = lowerLeft->axis( style, i_tstyle );
     }
 
-    double lmWidth = 0, lmHeight = 0, lmMidline = 0;
+    luPixel lmWidth = 0, lmHeight = 0, lmMidline = 0;
     if (hasLowerMiddle()) {
-        lowerMiddle->calcSizes(contextStyle,
-			       contextStyle.convertTextStyleIndex(tstyle),
-			       contextStyle.convertIndexStyleLower(istyle));
+        lowerMiddle->calcSizes( style, i_tstyle, l_istyle );
         lmWidth = lowerMiddle->getWidth();
         lmHeight = lowerMiddle->getHeight() + distY;
-        lmMidline = lowerMiddle->getMidline();
+        lmMidline = lowerMiddle->axis( style, i_tstyle );
     }
 
-    double lrWidth = 0, lrHeight = 0, lrMidline = 0;
+    luPixel lrWidth = 0, lrHeight = 0, lrMidline = 0;
     if (hasLowerRight()) {
-        lowerRight->calcSizes(contextStyle,
-			      contextStyle.convertTextStyleIndex(tstyle),
-			      contextStyle.convertIndexStyleLower(istyle));
+        lowerRight->calcSizes( style, i_tstyle, l_istyle );
         lrWidth = lowerRight->getWidth();
         lrHeight = lowerRight->getHeight();
-        lrMidline = lowerRight->getMidline();
+        lrMidline = lowerRight->axis( style, i_tstyle );
     }
 
     // get the contents size
-    content->calcSizes(contextStyle, tstyle, istyle);
-    double width = QMAX(content->getWidth(), QMAX(umWidth, lmWidth));
-    double toMidline = content->getMidline();
-    double fromMidline = content->getHeight() - toMidline;
+    content->calcSizes(style, tstyle, istyle);
+    luPixel width = QMAX(content->getWidth(), QMAX(umWidth, lmWidth));
+    luPixel toMidline = content->axis( style, tstyle );
+    luPixel fromMidline = content->getHeight() - toMidline;
 
     // calculate the x offsets
     if (ulWidth > llWidth) {
@@ -277,19 +423,19 @@ void IndexElement::calcSizes(const ContextStyle& contextStyle, ContextStyle::Tex
     width += QMAX(urWidth, lrWidth);
 
     // calculate the y offsets
-    double ulOffset = 0;
-    double urOffset = 0;
-    double llOffset = 0;
-    double lrOffset = 0;
+    luPixel ulOffset = 0;
+    luPixel urOffset = 0;
+    luPixel llOffset = 0;
+    luPixel lrOffset = 0;
     if (content->isTextOnly()) {
-        double mySize = contextStyle.getAdjustedSize( tstyle );
-        QFont font = contextStyle.getDefaultFont();
-        font.setPointSizeFloat(mySize);
+        luPt mySize = style.getAdjustedSize( tstyle );
+        QFont font = style.getDefaultFont();
+        font.setPointSizeFloat( style.layoutUnitPtToPt( mySize ) );
 
         QFontMetrics fm(font);
-        QRect bound = fm.boundingRect('x');
+        LuPixelRect bound = fm.boundingRect('x');
 
-        int exBaseline = -bound.top();
+        luPixel exBaseline = style.ptToLayoutUnitPt( -bound.top() );
 
         // the upper half
         ulOffset = ulHeight + exBaseline - content->getBaseline();
@@ -301,14 +447,14 @@ void IndexElement::calcSizes(const ContextStyle& contextStyle, ContextStyle::Tex
     else {
 
         // the upper half
-        ulOffset = QMAX(ulMidline, ulHeight-content->getMidline());
-        urOffset = QMAX(urMidline, urHeight-content->getMidline());
+        ulOffset = QMAX(ulMidline, ulHeight-toMidline);
+        urOffset = QMAX(urMidline, urHeight-toMidline);
 
         // the lower half
-        llOffset = QMAX(content->getHeight()-llMidline, content->getMidline());
-        lrOffset = QMAX(content->getHeight()-lrMidline, content->getMidline());
+        llOffset = QMAX(content->getHeight()-llMidline, toMidline);
+        lrOffset = QMAX(content->getHeight()-lrMidline, toMidline);
     }
-    double height = QMAX(umHeight, QMAX(ulOffset, urOffset));
+    luPixel height = QMAX(umHeight, QMAX(ulOffset, urOffset));
 
     // the upper half
     content->setY(height);
@@ -341,11 +487,11 @@ void IndexElement::calcSizes(const ContextStyle& contextStyle, ContextStyle::Tex
     setHeight(toMidline+fromMidline);
     if (content->isTextOnly()) {
         setBaseline(content->getY() + content->getBaseline());
-        setMidline(content->getY() + content->getMidline());
+        //setMidline(content->getY() + content->getMidline());
     }
     else {
-        setMidline(toMidline);
-        calcBaseline();
+        //setMidline(toMidline);
+        setBaseline(content->getBaseline() + content->getY());
     }
 }
 
@@ -354,46 +500,38 @@ void IndexElement::calcSizes(const ContextStyle& contextStyle, ContextStyle::Tex
  * The `parentOrigin' is the point this element's parent starts.
  * We can use our parentPosition to get our own origin then.
  */
-void IndexElement::draw(QPainter& painter, const QRect& r,
-                        const ContextStyle& contextStyle,
-			ContextStyle::TextStyle tstyle,
-			ContextStyle::IndexStyle istyle,
-                        const KoPoint& parentOrigin)
+void IndexElement::draw( QPainter& painter, const LuPixelRect& r,
+                         const ContextStyle& style,
+                         ContextStyle::TextStyle tstyle,
+                         ContextStyle::IndexStyle istyle,
+                         const LuPixelPoint& parentOrigin )
 {
-    KoPoint myPos(parentOrigin.x()+getX(), parentOrigin.y()+getY());
-    if (!QRect(myPos.x(), myPos.y(), getWidth(), getHeight()).intersects(r))
-        return;
+    LuPixelPoint myPos( parentOrigin.x()+getX(), parentOrigin.y()+getY() );
+    //if ( !LuPixelRect( myPos.x(), myPos.y(), getWidth(), getHeight() ).intersects( r ) )
+    //    return;
 
-    content->draw(painter, r, contextStyle, tstyle, istyle, myPos);
+    ContextStyle::TextStyle i_tstyle = style.convertTextStyleIndex(tstyle);
+    ContextStyle::IndexStyle u_istyle = style.convertIndexStyleUpper( istyle );
+    ContextStyle::IndexStyle l_istyle = style.convertIndexStyleLower( istyle );
+
+    content->draw(painter, r, style, tstyle, istyle, myPos);
     if (hasUpperLeft()) {
-        upperLeft->draw(painter, r, contextStyle,
-			contextStyle.convertTextStyleIndex(tstyle),
-			contextStyle.convertIndexStyleUpper(istyle), myPos);
+        upperLeft->draw(painter, r, style, i_tstyle, u_istyle, myPos);
     }
     if (hasUpperMiddle()) {
-        upperMiddle->draw(painter, r, contextStyle,
-			  contextStyle.convertTextStyleIndex(tstyle),
-			  contextStyle.convertIndexStyleUpper(istyle), myPos);
+        upperMiddle->draw(painter, r, style, i_tstyle, u_istyle, myPos);
     }
     if (hasUpperRight()) {
-        upperRight->draw(painter, r, contextStyle,
-			 contextStyle.convertTextStyleIndex(tstyle),
-			 contextStyle.convertIndexStyleUpper(istyle), myPos);
+        upperRight->draw(painter, r, style, i_tstyle, u_istyle, myPos);
     }
     if (hasLowerLeft()) {
-        lowerLeft->draw(painter, r, contextStyle,
-			contextStyle.convertTextStyleIndex(tstyle),
-			contextStyle.convertIndexStyleLower(istyle), myPos);
+        lowerLeft->draw(painter, r, style, i_tstyle, l_istyle, myPos);
     }
     if (hasLowerMiddle()) {
-        lowerMiddle->draw(painter, r, contextStyle,
-			  contextStyle.convertTextStyleIndex(tstyle),
-			  contextStyle.convertIndexStyleLower(istyle), myPos);
+        lowerMiddle->draw(painter, r, style, i_tstyle, l_istyle, myPos);
     }
     if (hasLowerRight()) {
-        lowerRight->draw(painter, r, contextStyle,
-			 contextStyle.convertTextStyleIndex(tstyle),
-			 contextStyle.convertIndexStyleLower(istyle), myPos);
+        lowerRight->draw(painter, r, style, i_tstyle, l_istyle, myPos);
     }
 
     // Debug
@@ -405,6 +543,30 @@ void IndexElement::draw(QPainter& painter, const QRect& r,
 }
 
 
+void IndexElement::dispatchFontCommand( FontCommand* cmd )
+{
+    content->dispatchFontCommand( cmd );
+    if (hasUpperLeft()) {
+        upperLeft->dispatchFontCommand( cmd );
+    }
+    if (hasUpperMiddle()) {
+        upperMiddle->dispatchFontCommand( cmd );
+    }
+    if (hasUpperRight()) {
+        upperRight->dispatchFontCommand( cmd );
+    }
+    if (hasLowerLeft()) {
+        lowerLeft->dispatchFontCommand( cmd );
+    }
+    if (hasLowerMiddle()) {
+        lowerMiddle->dispatchFontCommand( cmd );
+    }
+    if (hasLowerRight()) {
+        lowerRight->dispatchFontCommand( cmd );
+    }
+}
+
+
 // navigation
 //
 // The elements are responsible to handle cursor movement themselves.
@@ -703,7 +865,7 @@ void IndexElement::moveDown(FormulaCursor* cursor, BasicElement* from)
  * The list will be emptied but stays the property of the caller.
  */
 void IndexElement::insert(FormulaCursor* cursor,
-                          QList<BasicElement>& newChildren,
+                          QPtrList<BasicElement>& newChildren,
                           Direction direction)
 {
     SequenceElement* index = static_cast<SequenceElement*>(newChildren.take(0));
@@ -756,7 +918,7 @@ void IndexElement::insert(FormulaCursor* cursor,
  * The ownership of the list is passed to the caller.
  */
 void IndexElement::remove(FormulaCursor* cursor,
-                          QList<BasicElement>& removedChildren,
+                          QPtrList<BasicElement>& removedChildren,
                           Direction direction)
 {
     int pos = cursor->getPos();
@@ -811,9 +973,9 @@ void IndexElement::remove(FormulaCursor* cursor,
  * Moves the cursor to a normal place where new elements
  * might be inserted.
  */
-void IndexElement::normalize(FormulaCursor* cursor, BasicElement::Direction direction)
+void IndexElement::normalize(FormulaCursor* cursor, Direction direction)
 {
-    if (direction == BasicElement::beforeCursor) {
+    if (direction == beforeCursor) {
         content->moveLeft(cursor, this);
     }
     else {
@@ -1018,7 +1180,7 @@ void IndexElement::moveToLowerRight(FormulaCursor* cursor, Direction direction)
 /**
  * Appends our attributes to the dom element.
  */
-void IndexElement::writeDom(QDomElement& element)
+void IndexElement::writeDom(QDomElement element)
 {
     BasicElement::writeDom(element);
 
@@ -1064,7 +1226,7 @@ void IndexElement::writeDom(QDomElement& element)
  * Reads our attributes from the element.
  * Returns false if it failed.
  */
-bool IndexElement::readAttributesFromDom(QDomElement& element)
+bool IndexElement::readAttributesFromDom(QDomElement element)
 {
     if (!BasicElement::readAttributesFromDom(element)) {
         return false;
@@ -1083,10 +1245,8 @@ bool IndexElement::readContentFromDom(QDomNode& node)
         return false;
     }
 
-    delete content;
-    content = buildChild(node, "CONTENT");
-    if (content == 0) {
-        cerr << "Empty content in IndexElement.\n";
+    if ( !buildChild( content, node, "CONTENT" ) ) {
+        kdWarning( DEBUGID ) << "Empty content in IndexElement." << endl;
         return false;
     }
     node = node.nextSibling();
@@ -1103,33 +1263,33 @@ bool IndexElement::readContentFromDom(QDomNode& node)
              lowerLeftRead && lowerMiddleRead && lowerRightRead)) {
 
         if (!upperLeftRead && (node.nodeName().upper() == "UPPERLEFT")) {
-            upperLeft = buildChild(node, "UPPERLEFT");
-            upperLeftRead = upperLeft != 0;
+            upperLeftRead = buildChild( upperLeft=new SequenceElement( this ), node, "UPPERLEFT" );
+            if ( !upperLeftRead ) return false;
         }
 
         if (!upperMiddleRead && (node.nodeName().upper() == "UPPERMIDDLE")) {
-            upperMiddle = buildChild(node, "UPPERMIDDLE");
-            upperMiddleRead = upperMiddle != 0;
+            upperMiddleRead = buildChild( upperMiddle=new SequenceElement( this ), node, "UPPERMIDDLE" );
+            if ( !upperMiddleRead ) return false;
         }
 
         if (!upperRightRead && (node.nodeName().upper() == "UPPERRIGHT")) {
-            upperRight = buildChild(node, "UPPERRIGHT");
-            upperRightRead = upperRight != 0;
+            upperRightRead = buildChild( upperRight=new SequenceElement( this ), node, "UPPERRIGHT" );
+            if ( !upperRightRead ) return false;
         }
 
         if (!lowerLeftRead && (node.nodeName().upper() == "LOWERLEFT")) {
-            lowerLeft = buildChild(node, "LOWERLEFT");
-            lowerLeftRead = lowerLeft != 0;
+            lowerLeftRead = buildChild( lowerLeft=new SequenceElement( this ), node, "LOWERLEFT" );
+            if ( !lowerLeftRead ) return false;
         }
 
         if (!lowerMiddleRead && (node.nodeName().upper() == "LOWERMIDDLE")) {
-            lowerMiddle = buildChild(node, "LOWERMIDDLE");
-            lowerMiddleRead = lowerMiddle != 0;
+            lowerMiddleRead = buildChild( lowerMiddle=new SequenceElement( this ), node, "LOWERMIDDLE" );
+            if ( !lowerMiddleRead ) return false;
         }
 
         if (!lowerRightRead && (node.nodeName().upper() == "LOWERRIGHT")) {
-            lowerRight = buildChild(node, "LOWERRIGHT");
-            lowerRightRead = lowerRight != 0;
+            lowerRightRead = buildChild( lowerRight=new SequenceElement( this ), node, "LOWERRIGHT" );
+            if ( !lowerRightRead ) return false;
         }
 
         node = node.nextSibling();
@@ -1138,7 +1298,7 @@ bool IndexElement::readContentFromDom(QDomNode& node)
         lowerLeftRead || lowerMiddleRead || lowerRightRead;
 }
 
-ElementIndexPtr IndexElement::getIndex(int position)
+ElementIndexPtr IndexElement::getIndex( int position )
 {
     switch (position) {
 	case upperRightPos:
@@ -1197,4 +1357,114 @@ QString IndexElement::toLatex()
     return index;
 }
 
+QString IndexElement::formulaString()
+{
+    QString index = "(" + content->formulaString() + ")";
+    if ( hasLowerRight() ) {
+        index += "_(" + lowerRight->formulaString() + ")";
+    }
+    if ( hasUpperRight() ) {
+        index += "**(" + upperRight->formulaString() + ")";
+    }
+    return index;
+}
+
+void IndexElement::writeMathML( QDomDocument doc, QDomNode parent )
+{
+    QDomElement de;
+    QDomElement uo;
+    bool uoscripts = true;
+
+    if ( hasUpperMiddle() && hasLowerMiddle() )
+    {
+        uo = doc.createElement( "munderover" );
+        content->writeMathML( doc, uo ); // base
+        lowerMiddle->writeMathML( doc, uo );
+        upperMiddle->writeMathML( doc, uo );
+    }
+    else if ( hasUpperMiddle() )
+    {
+        uo = doc.createElement( "mover" );
+        content->writeMathML( doc, uo ); // base
+        upperMiddle->writeMathML( doc, uo );
+    }
+    else if ( hasLowerMiddle() )
+    {
+        uo = doc.createElement( "munder" );
+        content->writeMathML( doc, uo ); // base
+        lowerMiddle->writeMathML( doc, uo );
+    }
+    else // no over- or underscripts
+        uoscripts = false;
+
+
+    if ( hasLowerLeft() || hasUpperLeft() )
+    {
+        de = doc.createElement( "mmultiscripts" );
+        if ( !uoscripts ) // base
+            content->writeMathML( doc, de );
+        else
+            de.appendChild( uo );
+
+        if ( hasLowerRight() )
+            lowerRight->writeMathML( doc, de );
+        else
+            de.appendChild( doc.createElement( "none" ) );
+
+        if ( hasUpperRight() )
+            upperRight->writeMathML( doc, de );
+        else
+            de.appendChild( doc.createElement( "none" ) );
+
+        de.appendChild( doc.createElement( "mprescripts" ) );
+
+        if ( hasLowerLeft() )
+            lowerLeft->writeMathML( doc, de );
+        else
+            de.appendChild( doc.createElement( "none" ) );
+
+        if ( hasUpperLeft() )
+            upperLeft->writeMathML( doc, de );
+        else
+            de.appendChild( doc.createElement( "none" ) );
+    }
+    else if ( hasLowerRight() || hasUpperRight() )
+    {
+        if ( !hasUpperRight() )
+        {
+            de = doc.createElement( "msub" );
+            if ( !uoscripts ) // base
+                content->writeMathML( doc, de );
+            else
+                de.appendChild( uo );
+            lowerRight->writeMathML( doc, de );
+        }
+        else if ( !hasLowerRight() )
+        {
+            de = doc.createElement( "msup" );
+            if ( !uoscripts ) // base
+                content->writeMathML( doc, de );
+            else
+                de.appendChild( uo );
+            upperRight->writeMathML( doc, de );
+        }
+        else // both
+        {
+            de = doc.createElement( "msubsup" );
+            if ( !uoscripts ) // base
+                content->writeMathML( doc, de );
+            else
+                de.appendChild( uo );
+            lowerRight->writeMathML( doc, de );
+            upperRight->writeMathML( doc, de );
+        }
+    }
+    else // no mmultiscripts, msubsup, msub or msup
+        de = uo;
+
+
+    parent.appendChild( de );
+
+}
+
 KFORMULA_NAMESPACE_END
