@@ -22,20 +22,25 @@
 #include <koMainWindow.h>
 #include <koFrame.h>
 #include <KoViewIface.h>
+#include <koDocumentChild.h>
 
-#include <kinstance.h>
-#include <kstddirs.h>
+#include <klocale.h>
+#include <kglobal.h>
 #include <kdebug.h>
-#include <kparts/plugin.h>
+#include <kparts/partmanager.h>
 #include <kparts/event.h>
-#include <kstatusbar.h>
 #include <assert.h>
+#include <kstatusbar.h>
+#include <qapplication.h>
+
+#define DEBUGRECT(rc) (rc).x() << "," << (rc).y() << " " << (rc).width() << "x" << (rc).height()
 
 class KoViewPrivate
 {
 public:
   KoViewPrivate()
   {
+    m_inOperation = false;
     m_zoom = 1.0;
     m_children.setAutoDelete( true );
     m_manager = 0L;
@@ -51,7 +56,7 @@ public:
   QGuardedPtr<KoDocument> m_doc; // our KoDocument
   QGuardedPtr<KParts::PartManager> m_manager;
   double m_zoom;
-  QList<KoViewChild> m_children;
+  QPtrList<KoViewChild> m_children;
   QWidget *m_tempActiveWidget;
   KoViewIface *m_dcopObject;
   bool m_registered;  // are we registered at the part manager?
@@ -97,12 +102,13 @@ public:
       bool m_visible;  // true when the item has been added to the statusbar
   };
   QValueList<StatusBarItem> m_statusBarItems; // Our statusbar items
+  bool m_inOperation; //in the middle of an operation (no screen refreshing)?
 };
 
 KoView::KoView( KoDocument *document, QWidget *parent, const char *name )
  : QWidget( parent, name )
 {
-  ASSERT( document );
+  Q_ASSERT( document );
 
   //kdDebug(30003) << "KoView::KoView " << this << endl;
   d = new KoViewPrivate;
@@ -116,23 +122,54 @@ KoView::KoView( KoDocument *document, QWidget *parent, const char *name )
   connect( d->m_doc, SIGNAL( childChanged( KoDocumentChild * ) ),
            this, SLOT( slotChildChanged( KoDocumentChild * ) ) );
 
+  connect( d->m_doc, SIGNAL( sigBeginOperation() ),
+           this, SLOT( beginOperation() ) );
+
+  connect( d->m_doc, SIGNAL( sigEndOperation() ),
+           this, SLOT( endOperation() ) );
+
+
+  actionCollection()->setWidget( this );
   setupGlobalActions();
+  KActionCollection *coll = actionCollection();
+  /**** not needed anymore, according to David (Werner)
   QValueList<KAction*> docActions = document->actionCollection()->actions();
   QValueList<KAction*>::ConstIterator it = docActions.begin();
   QValueList<KAction*>::ConstIterator end = docActions.end();
   for (; it != end; ++it )
-    actionCollection()->insert( *it );
+      coll->insert( *it );
+  */
+
+  KStatusBar * sb = statusBar();
+  if ( sb ) // No statusbar in e.g. konqueror
+  {
+      coll->setHighlightingEnabled( true );
+      connect( coll, SIGNAL( actionStatusText( const QString & ) ),
+               this, SLOT( slotActionStatusText( const QString & ) ) );
+      connect( coll, SIGNAL( clearStatusText() ),
+               this, SLOT( slotClearStatusText() ) );
+
+      connect( d->m_doc, SIGNAL( sigStatusBarMessage( const QString& ) ),
+               this, SLOT( slotActionStatusText( const QString& ) ) );
+      connect( d->m_doc, SIGNAL( sigClearStatusBarMessage() ),
+               this, SLOT( slotClearStatusText() ) );
+  }
+  d->m_doc->setCurrent();
 }
 
 KoView::~KoView()
 {
   kdDebug(30003) << "KoView::~KoView " << this << endl;
   delete d->m_dcopObject;
-  if ( koDocument() && !koDocument()->isSingleViewMode() )
+  if (!d->m_documentDeleted)
   {
-    if ( d->m_manager && d->m_registered ) // if we aren't registered we mustn't unregister :)
-      d->m_manager->removePart( koDocument() );
-    d->m_doc->removeView(this);
+    if ( koDocument() && !koDocument()->isSingleViewMode() )
+    {
+      if ( d->m_manager && d->m_registered ) // if we aren't registered we mustn't unregister :)
+        d->m_manager->removePart( koDocument() );
+      d->m_doc->removeView(this);
+      d->m_doc->setCurrent( false );
+    }
   }
   delete d;
 }
@@ -303,7 +340,11 @@ void KoView::partActivateEvent( KParts::PartActivateEvent *event )
       {
         KoViewChild *viewChild = new KoViewChild( child, this );
         d->m_children.append( viewChild );
+
+        QApplication::setOverrideCursor(waitCursor);
+        // This is the long operation (due to toolbar layout stuff)
         d->m_manager->setActivePart( child->document(), viewChild->frame()->view() );
+        QApplication::restoreOverrideCursor();
       }
       else
       {
@@ -353,24 +394,21 @@ void KoView::partSelectEvent( KParts::PartSelectEvent *event )
 }
 
 void KoView::guiActivateEvent( KParts::GUIActivateEvent * ev )
+{
+    showAllStatusBarItems( ev->activated() );
+}
+
+void KoView::showAllStatusBarItems( bool show )
 {
     KStatusBar * sb = statusBar();
-    //kdDebug() << "KoView::guiActivateEvent activated=" << ev->activated()
-    //          << " sb=" << (void*)sb << endl;
     if ( !sb )
         return;
-    if ( ev->activated() )
-    {
-        QValueListIterator<KoViewPrivate::StatusBarItem> it = d->m_statusBarItems.begin();
-        for ( ; it != d->m_statusBarItems.end() ; ++it )
+    QValueListIterator<KoViewPrivate::StatusBarItem> it = d->m_statusBarItems.begin();
+    for ( ; it != d->m_statusBarItems.end() ; ++it )
+        if ( show )
             (*it).ensureItemShown( sb );
-    }
-    else
-    {
-        QValueListIterator<KoViewPrivate::StatusBarItem> it = d->m_statusBarItems.begin();
-        for ( ; it != d->m_statusBarItems.end() ; ++it )
+        else
             (*it).ensureItemHidden( sb );
-    }
 }
 
 void KoView::addStatusBarItem( QWidget * widget, int stretch, bool permanent )
@@ -379,7 +417,7 @@ void KoView::addStatusBarItem( QWidget * widget, int stretch, bool permanent )
     d->m_statusBarItems.append(item);
     QValueListIterator<KoViewPrivate::StatusBarItem> it = d->m_statusBarItems.fromLast();
     KStatusBar * sb = statusBar();
-    ASSERT(sb);
+    Q_ASSERT(sb);
     if (sb)
         (*it).ensureItemShown( sb );
 }
@@ -433,7 +471,7 @@ void KoView::paintEverything( QPainter &painter, const QRect &rect, bool transpa
 
 KoViewChild *KoView::child( KoView *view )
 {
-  QListIterator<KoViewChild> it( d->m_children );
+  QPtrListIterator<KoViewChild> it( d->m_children );
   for (; it.current(); ++it )
     if ( it.current()->frame()->view() == view )
       return it.current();
@@ -443,7 +481,7 @@ KoViewChild *KoView::child( KoView *view )
 
 KoViewChild *KoView::child( KoDocument *doc )
 {
-  QListIterator<KoViewChild> it( d->m_children );
+  QPtrListIterator<KoViewChild> it( d->m_children );
   for (; it.current(); ++it )
     if ( it.current()->documentChild()->document() == doc )
       return it.current();
@@ -490,17 +528,24 @@ void KoView::slotChildActivated( bool a )
 
 
   d->m_tempActiveWidget = activeWidget;
+  QApplication::setOverrideCursor(waitCursor);
   d->m_manager->setActivePart( 0L );
 
   QGuardedPtr<KoDocumentChild> docChild = ch->documentChild();
   QGuardedPtr<KoFrame> chFrame = ch->frame();
   if ( docChild && chFrame && chFrame->view() )
+  {
+    docChild->setContentsPos( chFrame->view()->canvasXOffset(),
+                              chFrame->view()->canvasYOffset() );
     docChild->document()->setViewBuildDocument( chFrame->view(), chFrame->view()->xmlguiBuildDocument() );
+  }
 
   d->m_children.remove( ch );
 
   d->m_manager->addPart( docChild->document(), false ); // the destruction of the view removed the part from the partmanager. re-add it :)
 
+  QApplication::restoreOverrideCursor();
+
   // #### HACK
   // We want to delete as many views as possible and this
   // trick is used to go upwards in the view-tree.
@@ -538,17 +583,55 @@ void KoView::newView() {
     shell->show();
 }
 
+bool KoView::isInOperation() const
+{
+   return d->m_inOperation;
+}
+
+void KoView::beginOperation()
+{
+   d->m_inOperation = true;
+   canvas()->setUpdatesEnabled(FALSE);
+}
+
+void KoView::endOperation()
+{
+   canvas()->setUpdatesEnabled(TRUE);
+   d->m_inOperation = false;
+
+//   canvas()->update();
+}
+
 KoMainWindow * KoView::shell() const
 {
     return dynamic_cast<KoMainWindow *>( topLevelWidget() );
 }
 
+KMainWindow * KoView::mainWindow() const
+{
+    return dynamic_cast<KMainWindow *>( topLevelWidget() );
+}
+
 KStatusBar * KoView::statusBar() const
 {
     KoMainWindow *mw = shell();
     return mw ? mw->statusBar() : 0L;
 }
 
+void KoView::slotActionStatusText( const QString &text )
+{
+  KStatusBar *sb = statusBar();
+  if ( sb )
+      sb->message( text );
+}
+
+void KoView::slotClearStatusText()
+{
+  KStatusBar *sb = statusBar();
+  if ( sb )
+      sb->clear();
+}
+
 DCOPObject *KoView::dcopObject()
 {
     if ( !d->m_dcopObject )
@@ -611,10 +694,11 @@ KoViewChild::KoViewChild( KoDocumentChild *child, KoView *_parentView )
   // This initial calculation uses R1 but omits borders because the frame is currently inactive (-> 0)
   QRect geom = child->geometry();
   double zoom = parentView()->zoom();
-  m_frame->setGeometry( static_cast<int>((double)geom.x() * zoom) - parentView()->canvasXOffset(),
-                        static_cast<int>((double)geom.y() * zoom) - parentView()->canvasYOffset(),
-                        static_cast<int>((double)geom.width() * zoom),
-                        static_cast<int>((double)geom.height() * zoom) );
+  // cast to int gives rounding probs, so we add 0.5 to fix it.
+  m_frame->setGeometry( static_cast<int>( (double)geom.x() * zoom + 0.5 ) - parentView()->canvasXOffset(),
+                        static_cast<int>((double)geom.y() * zoom + 0.5) - parentView()->canvasYOffset(),
+                        static_cast<int>((double)geom.width() * zoom + 0.5),
+                        static_cast<int>((double)geom.height() * zoom + 0.5) );
 
   m_frame->show();
   m_frame->raise();
@@ -639,8 +723,6 @@ KoViewChild::~KoViewChild()
   delete d;
 }
 
-#define DEBUGRECT(rc) (rc).x() << "," << (rc).y() << " " << (rc).width() << "x" << (rc).height()
-
 void KoViewChild::slotFrameGeometryChanged()
 {
   // Set our geometry from the frame geometry (R2 reversed)
@@ -660,10 +742,10 @@ void KoViewChild::slotFrameGeometryChanged()
                           geom.width() - m_frame->leftBorder() - m_frame->rightBorder(),
                           geom.height() - m_frame->topBorder() - m_frame->bottomBorder() );
     double zoom = parentView()->zoom();
-    QRect unzoomedRect( static_cast<int>( (double)borderLessRect.x() / zoom ),
-                        static_cast<int>( (double)borderLessRect.y() / zoom ),
-                        static_cast<int>( (double)borderLessRect.width() / zoom ),
-                        static_cast<int>( (double)borderLessRect.height() / zoom ) );
+    QRect unzoomedRect( static_cast<int>( (double)borderLessRect.x() / zoom + 0.5 ),
+                        static_cast<int>( (double)borderLessRect.y() / zoom + 0.5 ),
+                        static_cast<int>( (double)borderLessRect.width() / zoom + 0.5 ),
+                        static_cast<int>( (double)borderLessRect.height() / zoom + 0.5 ) );
     kdDebug() << "KoViewChild::slotFrameGeometryChanged child geometry "
               << ( m_child->geometry() == unzoomedRect ? "already " : "set to " )
               << DEBUGRECT( unzoomedRect ) << endl;
@@ -683,10 +765,10 @@ void KoViewChild::slotDocGeometryChanged()
   // The frame's resizeEvent will call slotFrameGeometryChanged.
   double zoom = parentView()->zoom();
   QRect geom = m_child->geometry();
-  QRect borderRect( static_cast<int>( (double)geom.x() * zoom ) - m_frame->leftBorder() - parentView()->canvasXOffset(),
-                    static_cast<int>( (double)geom.y() * zoom ) - m_frame->topBorder() - parentView()->canvasYOffset(),
-                    static_cast<int>( (double)geom.width() * zoom ) + m_frame->leftBorder() + m_frame->rightBorder(),
-                    static_cast<int>( (double)geom.height() * zoom ) + m_frame->topBorder() + m_frame->bottomBorder() );
+  QRect borderRect( static_cast<int>( (double)geom.x() * zoom + 0.5 ) - m_frame->leftBorder() - parentView()->canvasXOffset(),
+                    static_cast<int>( (double)geom.y() * zoom + 0.5 ) - m_frame->topBorder() - parentView()->canvasYOffset(),
+                    static_cast<int>( (double)geom.width() * zoom + 0.5 ) + m_frame->leftBorder() + m_frame->rightBorder(),
+                    static_cast<int>( (double)geom.height() * zoom + 0.5 ) + m_frame->topBorder() + m_frame->bottomBorder() );
   kdDebug() << "KoViewChild::slotDocGeometryChanged frame geometry "
             << ( m_frame->geometry() == borderRect ? "already " : "set to " )
             << DEBUGRECT( borderRect ) << endl;
