@@ -1,3 +1,22 @@
+/* This file is part of the KDE project
+   Copyright (C) 1998, 1999, 2000 Torben Weis <weis@kde.org>
+
+   This library is free software; you can redistribute it and/or
+   modify it under the terms of the GNU Library General Public
+   License as published by the Free Software Foundation; either
+   version 2 of the License, or (at your option) any later version.
+
+   This library is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   Library General Public License for more details.
+
+   You should have received a copy of the GNU Library General Public License
+   along with this library; see the file COPYING.LIB.  If not, write to
+   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+   Boston, MA 02111-1307, USA.
+*/
+
 #include "koscript_eval.h"
 #include "koscript_value.h"
 #include "koscript_context.h"
@@ -80,7 +99,7 @@ bool KSEval_definitions( KSParseNode* node, KSContext& context )
   {
     if ( node->branch1()->getType() == func_dcl )
     {
-      ASSERT( context.scope() );
+      Q_ASSERT( context.scope() );
       context.scope()->addObject( node->branch1()->getIdent(), new KSValue( new KSScriptFunction( context.scope()->module(), node->branch1() ) ) );
     }
     else if ( !node->branch1()->eval( context ) )
@@ -90,7 +109,7 @@ bool KSEval_definitions( KSParseNode* node, KSContext& context )
   {
     if ( node->branch2()->getType() == func_dcl )
     {
-      ASSERT( context.scope() );
+      Q_ASSERT( context.scope() );
       context.scope()->addObject( node->branch2()->getIdent(), new KSValue( new KSScriptFunction( context.scope()->module(), node->branch2() ) ) );
     }
     else if ( !node->branch2()->eval( context ) )
@@ -102,7 +121,7 @@ bool KSEval_definitions( KSParseNode* node, KSContext& context )
 
 bool KSEval_exports( KSParseNode* node, KSContext& context )
 {
-  ASSERT( context.value() );
+  Q_ASSERT( context.value() );
 
   if ( context.value()->type() == KSValue::StructClassType )
   {
@@ -122,7 +141,7 @@ bool KSEval_exports( KSParseNode* node, KSContext& context )
     }
   }
   else
-    ASSERT( 0 );
+    Q_ASSERT( 0 );
 
   return true;
 }
@@ -214,10 +233,16 @@ bool KSEval_t_plus_sign( KSParseNode* node, KSContext& context )
   {
     if ( !node->branch1()->eval( context ) )
       return false;
-    if ( context.value()->cast( KSValue::IntType ) )
-      return true;
-    if ( context.value()->cast( KSValue::DoubleType ) )
+
+    /* this operator is valid on numeric types so if it casts to a double it's ok.
+       I'm not sure if there is possible data loss converting from int to double
+       so avoid casting to double in that case just to be safe.
+    */
+    if ( context.value()->type() == KSValue::IntType ||
+         context.value()->cast( KSValue::DoubleType) )
+    {
       return true;
+    }
 
     QString tmp( i18n("Unary Operator + not defined for type %1") );
     context.setException( new KSException( "UnknownOperation", tmp.arg( context.value()->typeName() ), node->getLineNo() ) );
@@ -251,6 +276,11 @@ bool KSEval_t_plus_sign( KSParseNode* node, KSContext& context )
       return TRUE;
   }
 
+  // allows addition of boolean value, such as True+False
+  if ( r.value()->type() == KSValue::BoolType )
+  {
+      r.value()->cast( KSValue::IntType );
+  }
 
   if ( l.value()->type() == KSValue::TimeType )
   {
@@ -272,6 +302,13 @@ bool KSEval_t_plus_sign( KSParseNode* node, KSContext& context )
       context.value()->setValue( d );
       return TRUE;
   }
+
+  // allows addition of boolean value, such as True+False
+  if ( l.value()->type() == KSValue::BoolType )
+  {
+      l.value()->cast( KSValue::IntType );
+  }
+
   // If we have double and int, then always convert to double
   else if ( l.value()->type() == KSValue::DoubleType )
   {
@@ -446,7 +483,7 @@ bool KSEval_t_minus_sign( KSParseNode* node, KSContext& context )
       if ( !KSUtil::checkType( context, r.value(), KSValue::IntType, true ) )
           return false;
       QDate d = l.value()->dateValue();
-      d = d.addDays( -r.value()->intValue() );
+   d = d.addDays( -r.value()->intValue() );
       FILL_VALUE( context, l, r );
       context.value()->setValue( d );
       return TRUE;
@@ -511,9 +548,14 @@ bool KSEval_t_asterik( KSParseNode* node, KSContext& context )
     {
     case KSValue::IntType:
       {
-        KScript::Long result = r.value()->intValue() * l.value()->intValue();
+        // promote to double multiplication( fix bug #42499 )
+        // hold it in long integer only if it is small enough
+        KScript::Double v = r.value()->doubleValue() * l.value()->doubleValue();
         FILL_VALUE( context, l, r );
-        context.value()->setValue( result );
+        if( fabs( v ) < 1e9 )
+          context.value()->setValue( (long) v );
+        else
+          context.value()->setValue( v );
         return true;
       }
     case KSValue::DoubleType:
@@ -660,7 +702,7 @@ bool KSEval_scoped_name( KSParseNode* node, KSContext& context )
 
 bool KSEval_const_dcl( KSParseNode* node, KSContext& context )
 {
-  ASSERT( node->branch1() );
+  Q_ASSERT( node->branch1() );
 
   KSContext( l );
   if ( !node->branch1()->eval( l ) )
@@ -674,7 +716,7 @@ bool KSEval_const_dcl( KSParseNode* node, KSContext& context )
   else if ( context.value()->type() == KSValue::StructClassType )
     context.value()->structClassValue()->nameSpace()->insert( node->getIdent(), l.shareValue() );
   else
-    ASSERT( 0 );
+    Q_ASSERT( 0 );
 
   return true;
 }
@@ -697,8 +739,8 @@ bool KSEval_func_dcl( KSParseNode* node, KSContext& context )
   // Are parameters left ?
   if ( !context.value()->listValue().isEmpty() )
   {
-    QString tmp( i18n("%1 arguments are not needed") );
-    context.setException( new KSException( "TooManyArguments", tmp.arg( context.value()->listValue().count() ), node->getLineNo() ) );
+    const QString tmp( i18n("1 argument is not needed", "%n arguments are not needed", context.value()->listValue().count() ) );
+    context.setException( new KSException( "TooManyArguments", tmp, node->getLineNo() ) );
     context.scope()->popLocalScope();
     return false;
   }
@@ -799,14 +841,21 @@ bool KSEval_t_equal( KSParseNode* node, KSContext& context )
 {
   EVAL_OPS( context, l, r, false );
 
+  KScript::Boolean result;
   if ( !r.value()->cast( l.value()->type() ) )
   {
-    QString tmp( i18n("From %1 to %2") );
-    context.setException( new KSException( "CastingError", tmp.arg( r.value()->typeName() ).arg( l.value()->typeName() ), node->getLineNo() ) );
-    return false;
+    /* If we can't cast the values to match, then they definitely aren't
+       equal.  Don't return a parse error here -- our users aren't expected
+       to have detailed understanding of the syntax.
+    */
+    result = false;
+  }
+  else
+  {
+
+    result = ( r.value()->cmp( *l.value() ) );
   }
 
-  KScript::Boolean result = ( r.value()->cmp( *l.value() ) );
   FILL_VALUE( context, l, r );
   context.value()->setValue( result );
   return true;
@@ -1068,7 +1117,7 @@ bool KSEval_t_dict( KSParseNode* node, KSContext& context )
     //    we can not provide a left expression
     else
     {
-      context.setException( new KSException( "NoLeftExpr", i18n("Expected a left expression"), node->getLineNo() ) );
+      context.setException( new KSException( "NoLeftExpr", i18n("Expected a left expression."), node->getLineNo() ) );
       return false;
     }
   }
@@ -1229,6 +1278,7 @@ bool KSEval_t_func_call( KSParseNode* node, KSContext& context )
 
   // Create a list of parameters
   context.setValue( new KSValue( KSValue::ListType ) );
+  context.setExtraData(new KSValue(KSValue::ListType));
 
   KSParseNode *right = node->branch2();
   if ( right )
@@ -1239,7 +1289,7 @@ bool KSEval_t_func_call( KSParseNode* node, KSContext& context )
   KSSubScope* scope = context.scope()->popLocalScope();
   KSModule* module = context.scope()->popModule();
 
-  bool b = FALSE;;
+  bool b = FALSE;
   if ( l.value()->cast( KSValue::FunctionType ) )
   {
     context.scope()->pushModule( l.value()->functionValue()->module() );
@@ -1262,7 +1312,7 @@ bool KSEval_t_func_call( KSParseNode* node, KSContext& context )
     context.scope()->popModule();
   }
   else
-    ASSERT( 0 );
+    Q_ASSERT( 0 );
 
   // Resume namespaces
   context.scope()->pushLocalScope( scope );
@@ -1281,7 +1331,7 @@ bool KSEval_t_func_call( KSParseNode* node, KSContext& context )
 bool KSEval_member_expr( KSParseNode* node, KSContext& context )
 {
   KSParseNode *left = node->branch1();
-  ASSERT( left );
+  Q_ASSERT( left );
 
   // This resets leftExpr to FALSE
   KSContext l( context );
@@ -1327,7 +1377,7 @@ bool KSEval_member_expr( KSParseNode* node, KSContext& context )
             l.scope()->popModule();
         }
         else
-            ASSERT( 0 );
+            Q_ASSERT( 0 );
 
         // Resume namespaces
         l.scope()->pushLocalScope( scope );
@@ -1672,7 +1722,7 @@ bool KSEval_t_decr( KSParseNode* node, KSContext& context )
 
   if ( context.value()->mode() != KSValue::LeftExpr )
   {
-    context.setException( new KSException( "NoLeftExpr", i18n("Expected a left expression in assignment"), node->getLineNo() ) );
+    context.setException( new KSException( "NoLeftExpr", i18n("Expected a left expression in assignment."), node->getLineNo() ) );
     return false;
   }
 
@@ -1927,7 +1977,7 @@ bool KSEval_t_subst( KSParseNode* node, KSContext& context )
 
     if ( l.value()->mode() != KSValue::LeftExpr )
     {
-        context.setException( new KSException( "NoLeftExpr", i18n("Expected a left expression in substitute"), node->getLineNo() ) );
+        context.setException( new KSException( "NoLeftExpr", i18n("Expected a left expression in substitute."), node->getLineNo() ) );
         return false;
     }
 
@@ -1935,7 +1985,7 @@ bool KSEval_t_subst( KSParseNode* node, KSContext& context )
         return FALSE;
 
     int pos = node->getIdent().find( '/' );
-    ASSERT( pos != -1 );
+    Q_ASSERT( pos != -1 );
     QString match = node->getIdent().left( pos );
     QString subst = node->getIdent().mid( pos + 1 );
     KRegExp* exp = context.interpreter()->regexp();
@@ -2009,6 +2059,15 @@ bool KSEval_func_call_params( KSParseNode* node, KSContext& context )
 
   context.value()->listValue().append( l.shareValue() );
 
+  if (left->getType() == t_cell || left->getType() == t_range)
+  {
+    context.extraData()->listValue().append(new KSValue(left->getStringLiteral()));
+  }
+  else
+  {
+    context.extraData()->listValue().append(new KSValue());
+  }
+
   // More parameters ?
   KSParseNode *right = node->branch2();
   if ( right )
@@ -2067,8 +2126,8 @@ bool KSEval_destructor_dcl( KSParseNode* node, KSContext& context )
   // Are parameters left ?
   if ( !context.value()->listValue().isEmpty() )
   {
-    QString tmp( i18n("%1 arguments are not needed") );
-    context.setException( new KSException( "TooManyArguments", tmp.arg( context.value()->listValue().count() ), node->getLineNo() ) );
+    const QString tmp( i18n("1 argument is not needed", "%n arguments are not needed", context.value()->listValue().count() ) );
+    context.setException( new KSException( "TooManyArguments", tmp, node->getLineNo() ) );
     context.scope()->popLocalScope();
     return false;
   }
@@ -2124,7 +2183,7 @@ bool KSEval_t_struct( KSParseNode* node, KSContext& context )
 
 bool KSEval_t_struct_members( KSParseNode* node, KSContext& context )
 {
-  ASSERT( context.value() && context.value()->type() == KSValue::StructClassType );
+  Q_ASSERT( context.value() && context.value()->type() == KSValue::StructClassType );
 
   context.value()->structClassValue()->addVariable( node->getIdent() );
 
@@ -2138,7 +2197,7 @@ bool KSEval_t_struct_members( KSParseNode* node, KSContext& context )
 
 extern bool KSEval_t_qualified_names( KSParseNode* node, KSContext& context )
 {
-  ASSERT( context.value() && context.value()->type() == KSValue::ListType );
+  Q_ASSERT( context.value() && context.value()->type() == KSValue::ListType );
 
   KSParseNode *left = node->branch1();
   if ( !left )
@@ -2187,7 +2246,7 @@ extern bool KSEval_t_try( KSParseNode* node, KSContext& context )
 
   // Execute the questionable code
   KSParseNode *left = node->branch1();
-  ASSERT( left );
+  Q_ASSERT( left );
   // No error -> Return
   if ( left->eval( context ) )
   {
@@ -2201,7 +2260,7 @@ extern bool KSEval_t_try( KSParseNode* node, KSContext& context )
 
   // Execute the catch clauses
   KSParseNode *right = node->branch2();
-  ASSERT( right );
+  Q_ASSERT( right );
   return right->eval( context );
 }
 
@@ -2211,7 +2270,7 @@ extern bool KSEval_t_catch( KSParseNode* node, KSContext& context )
 
   // Find the type to which we want to compare
   KSParseNode *left = node->branch1();
-  ASSERT( left );
+  Q_ASSERT( left );
   if ( !left->eval( d ) )
   {
     context.setException( d );
@@ -2219,7 +2278,7 @@ extern bool KSEval_t_catch( KSParseNode* node, KSContext& context )
   }
 
   // Exception of the correct type ?
-  ASSERT( context.exception() );
+  Q_ASSERT( context.exception() );
   if ( context.exception()->type()->cmp( *d.value() ) )
   {
      // Get infos about the exception
@@ -2231,12 +2290,12 @@ extern bool KSEval_t_catch( KSParseNode* node, KSContext& context )
     nspace.insert( node->getIdent(), new KSValue( *value ) );
     context.scope()->localScope()->pushNamespace( &nspace );
 
-    // Clear the exception since we catched it
+    // Clear the exception since we caught it
     context.setException( 0 );
 
     // Evaluate the catch code
     KSParseNode *right = node->branch2();
-    ASSERT( right );
+    Q_ASSERT( right );
 
     /* bool res = */ right->eval( context );
 
@@ -2262,7 +2321,7 @@ extern bool KSEval_t_catch_default( KSParseNode* node, KSContext& context )
   // Find out na,me of the variable that
   // holds the type
   KSParseNode *left = node->branch1();
-  ASSERT( left );
+  Q_ASSERT( left );
   QString name1 = left->getIdent();
 
   // Clear the exception
@@ -2280,7 +2339,7 @@ extern bool KSEval_t_catch_default( KSParseNode* node, KSContext& context )
 
   // Evaluate the catch code
   KSParseNode *right = node->branch2();
-  ASSERT( right );
+  Q_ASSERT( right );
   bool res = right->eval( context );
 
   context.scope()->localScope()->popNamespace();
@@ -2370,7 +2429,7 @@ bool KSEval_plus_assign( KSParseNode* node, KSContext& context )
 
     if ( l.value()->mode() != KSValue::LeftExpr )
     {
-        context.setException( new KSException( "NoLeftExpr", i18n("Expected a left expression in assignment"), node->getLineNo() ) );
+        context.setException( new KSException( "NoLeftExpr", i18n("Expected a left expression in assignment."), node->getLineNo() ) );
         return false;
     }
 
@@ -2439,7 +2498,7 @@ bool KSEval_minus_assign( KSParseNode* node, KSContext& context )
 
     if ( l.value()->mode() != KSValue::LeftExpr )
     {
-        context.setException( new KSException( "NoLeftExpr", i18n("Expected a left expression in assignment"), node->getLineNo() ) );
+        context.setException( new KSException( "NoLeftExpr", i18n("Expected a left expression in assignment."), node->getLineNo() ) );
         return false;
     }
 
