@@ -17,6 +17,11 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <qglobal.h>
+#if QT_VERSION >= 0x030200
+#define INDIC
+#endif
+
 #include "kotextformatter.h"
 #include "kotextformat.h"
 #include "kotextdocument.h"
@@ -34,6 +39,9 @@
 // Line and paragraph width
 //#define DEBUG_FORMATTER_WIDTH
 
+// Hyphenation
+//#define DEBUG_HYPHENATION
+
 /////// keep in sync with kotextformat.cc !
 //#define REF_IS_LU
 
@@ -114,27 +122,42 @@ QPair<int, int> KoTextFormatterCore::determineCharWidth()
 }
 
 
+int KoTextFormatterCore::leftMargin( bool firstLine ) const
+{
+    int left = /*doc ?*/ parag->leftMargin() + doc->leftMargin() /*: 0*/;
+    if ( firstLine && !parag->string()->isRightToLeft() )
+    {
+        left += parag->firstLineMargin();
+        // Add the width of the paragraph counter - first line of parag only.
+        if( parag->counter() &&
+            ( parag->counter()->alignment() == Qt::AlignLeft ||
+              parag->counter()->alignment() == Qt::AlignAuto ) )
+            left += parag->counterWidth(); // in LU pixels
+    }
+    return left;
+}
+
+int KoTextFormatterCore::rightMargin( bool firstLine ) const
+{
+    int right = parag->rightMargin(); // 'rm' in QRT
+    if ( /*doc &&*/ firstLine && parag->string()->isRightToLeft() )
+        right += parag->firstLineMargin();
+    return right;
+}
+
 bool KoTextFormatterCore::format()
 {
     start = 0; // we don't do partial formatting yet
+    KoTextString *string = parag->string();
     if ( start == 0 )
-        c = &parag->string()->at( start );
+        c = &string->at( start );
     else
         c = 0;
 
     KoTextStringChar *firstChar = 0;
-    KoTextString *string = parag->string();
-    bool rtl = string->isRightToLeft();
     int left = doc ? parag->leftMargin() + doc->leftMargin() : 0;
-    x = left;
-    if ( doc && !rtl )
-        x += parag->firstLineMargin();
-    // Add the width of the paragraph counter - first line of parag only.
-    if( parag->counter() && !rtl &&
-    	(( parag->counter()->alignment() == Qt::AlignLeft ) || ( parag->counter()->alignment() == Qt::AlignAuto )))
-        x += parag->counterWidth(); // in LU pixels
+    int initialLMargin = leftMargin( true );
 
-    int curLeft = left;
     y = doc && doc->addMargins() ? parag->topMargin() : 0;
     // #57555, top margin doesn't apply if parag at top of page
     // (but a portion of the margin can be needed, to complete the prev page)
@@ -160,15 +183,17 @@ bool KoTextFormatterCore::format()
 
     int initialHeight = c->height(); // remember what adjustMargins was called with
 
-    int currentRightMargin = parag->rightMargin(); // 'rm' in QRT
-    if ( doc && rtl )
-        currentRightMargin += parag->firstLineMargin();
+    int currentRightMargin = rightMargin( true );
     int initialRMargin = currentRightMargin;
-    // Those two things must be done before calling determineCharWidth
+    // Those three things must be done before calling determineCharWidth
     i = start;
     parag->tabCache().clear();
+    x = 0;
 
     // We need the width of the first char for adjustMargins
+    // The result might not be 100% accurate when using a tab (it'll use x=0
+    // but with counters/margins this might be different). This is why
+    // we call determineCharWidth() again from within the loop.
     QPair<int, int> widths = determineCharWidth();
     int ww = widths.first; // width in layout units
 #ifndef REF_IS_LU
@@ -179,12 +204,12 @@ bool KoTextFormatterCore::format()
     // We are in a variable-width design, so it is returned by each call to adjustMargins.
     int dw = 0;
     //if (doc) // always true in kotext
-    doc->flow()->adjustMargins( y + parag->rect().y(), initialHeight,
-                                ww, x, initialRMargin, dw, parag );
+    doc->flow()->adjustMargins( y + parag->rect().y(), initialHeight, // input params
+                                ww, initialLMargin, initialRMargin, dw,  // output params
+                                parag );
     //else dw = parag->documentVisibleWidth();
 
-    int initialLMargin = x;
-    curLeft = x;
+    x = initialLMargin; // as modified by adjustMargins
 
     int maxY = doc ? doc->flow()->availableHeight() : -1;
 
@@ -195,7 +220,7 @@ bool KoTextFormatterCore::format()
                    << " left=" << left << " initialHeight=" << initialHeight << " initialLMargin=" << initialLMargin << " initialRMargin=" << initialRMargin << " availableWidth=" << availableWidth << " maxY=" << maxY << endl;
 #else
     if ( availableWidth == 0 )
-        kdDebug(32500) << "KoTextFormatter::format " << parag->paragId() << " warning, availableWidth=0" << endl;;
+        kdDebug(32500) << "KoTextFormatter::format " << parag->paragId() << " warning, availableWidth=0" << endl;
     if ( maxY == 0 )
         kdDebug(32500) << "KoTextFormatter::format " << parag->paragId() << " warning, maxY=0" << endl;
 #endif
@@ -243,7 +268,7 @@ bool KoTextFormatterCore::format()
         if ( c )
             lastChr = c;
         c = &string->at( i );
-        if ( i > 0 && (x > curLeft || ww == 0) || lastWasNonInlineCustom ) {
+        if ( i > 0 && (x > initialLMargin || ww == 0) || lastWasNonInlineCustom ) {
             c->lineStart = 0;
         } else {
             c->lineStart = 1;
@@ -260,12 +285,9 @@ bool KoTextFormatterCore::format()
         else
             lastWasNonInlineCustom = FALSE;
 
-        if ( i > 0 ) // first one was done above already
-        {
-            QPair<int, int> widths = determineCharWidth();
-            ww = widths.first;
-            pixelww = widths.second;
-        }
+        QPair<int, int> widths = determineCharWidth();
+        ww = widths.first;
+        pixelww = widths.second;
 
         // We're "aborting" the formatting. This still means we need to set the
         // lineStart bools to false (trouble ahead, otherwise!), and while we're at
@@ -352,7 +374,7 @@ bool KoTextFormatterCore::format()
                 {
                     QString lang = string->at(wordStart).format()->language();
                     char * hyphens = settings->hyphenator()->hyphens( word, lang );
-#ifdef DEBUG_FORMATTER
+#if defined(DEBUG_FORMATTER) || defined(DEBUG_HYPHENATION)
                     kdDebug(32500) << "Hyphenation: word=" << word << " lang=" << lang << " hyphens=" << hyphens << " maxlen=" << maxlen << endl;
                     kdDebug(32500) << "Parag indexes: wordStart=" << wordStart << " lastBreak=" << lastBreak << " i=" << i << endl;
 #endif
@@ -370,7 +392,7 @@ bool KoTextFormatterCore::format()
                             lineStart->hyphenated = true;
                             lastBreak = hypos + wordStart;
                             hyphenated = true;
-#if defined(DEBUG_FORMATTER) || defined(DEBUG_FORMATTER_WIDTH)
+#if defined(DEBUG_FORMATTER) || defined(DEBUG_FORMATTER_WIDTH) || defined(DEBUG_HYPHENATION)
                             kdDebug(32500) << "Hyphenation: will break at " << lastBreak << " using tempworddata at position " << hypos << "/" << tempWordData.size() << endl;
 #endif
                             if ( hypos < (int)tempWordData.size() )
@@ -461,7 +483,6 @@ bool KoTextFormatterCore::format()
                     }
                     if ( x != left || availableWidth != dw )
                         fullWidth = FALSE;
-                    curLeft = x;
                     lineStart->y = y;
                     parag->insertLineStart( i, lineStart );
                     tempWordData.clear();
@@ -553,7 +574,6 @@ bool KoTextFormatterCore::format()
                     availableWidth = dw - initialRMargin;
                     if ( x != left || availableWidth != dw )
                         fullWidth = FALSE;
-                    curLeft = x;
                     lineStart->y = y;
                     parag->insertLineStart( i + 1, lineStart );
                     tempWordData.clear();
@@ -608,20 +628,17 @@ bool KoTextFormatterCore::format()
             // format this line again
             if ( doc && lineStart->h > initialHeight )
             {
-                int lm = left + ( ( firstChar == &string->at(0) && doc && !rtl ) ? parag->firstLineMargin() : 0 );
-#ifdef DEBUG_FORMATTER
-                kdDebug(32500) << "left=" << left << ", firstlinepargin=" << (( firstChar == &string->at(0) && doc ) ? parag->firstLineMargin() : 0) << " => lm=" << lm << endl;
-#endif
-                int newLMargin = lm;
-                int newRMargin = currentRightMargin - ( ( firstChar == &string->at(0) && doc && rtl ) ? parag->firstLineMargin() : 0 );
+                bool firstLine = ( firstChar == &string->at( 0 ) );
+                int newLMargin = leftMargin( firstLine );
+                int newRMargin = rightMargin( firstLine );
                 int newPageWidth = dw;
                 initialHeight = lineStart->h;
                 doc->flow()->adjustMargins( y + parag->rect().y(), initialHeight,
-                                            ww,
+                                            firstChar->width,
                                             newLMargin, newRMargin, newPageWidth, parag );
 
 #ifdef DEBUG_FORMATTER
-                kdDebug(32500) << "new height: " << initialHeight << " => left=" << left << " lm=" << lm << " first-char=" << (firstChar==&string->at(0)) << " newLMargin=" << newLMargin << " newRMargin=" << newRMargin << endl;
+                kdDebug(32500) << "new height: " << initialHeight << " => left=" << left << " first-char=" << (firstChar==&string->at(0)) << " newLMargin=" << newLMargin << " newRMargin=" << newRMargin << endl;
 #endif
                 if ( newLMargin != initialLMargin || newRMargin != initialRMargin || newPageWidth != dw )
                 {
@@ -640,7 +657,6 @@ bool KoTextFormatterCore::format()
                     tmpBaseLine = c->ascent();
                     lineStart->h = tmph;
                     lineStart->baseLine = tmpBaseLine;
-                    curLeft = x;
                     lastBreak = -1;
                     col = 0;
                     //minw = x;
@@ -814,6 +830,18 @@ KoTextParagLineStart *KoTextFormatterCore::koFormatLine(
     space = QMAX( space, 0 ); // #### with nested tables this gets negative because of a bug I didn't find yet, so workaround for now. This also means non-left aligned nested tables do not work at the moment
     int start = (startChar - &string->at(0));
     int last = (lastChar - &string->at(0) );
+#ifdef INDIC
+
+    KoTextStringChar *ch = lastChar;
+    while ( ch > startChar && ch->whiteSpace ) {
+        space += ch->format()->width( ' ' );
+        --ch;
+    }
+
+    if (space < 0)
+        space = 0;
+
+#endif
     // do alignment Auto == Left in this case
     if ( align & Qt::AlignHCenter || align & Qt::AlignRight ) {
         if ( align & Qt::AlignHCenter )
@@ -911,6 +939,18 @@ KoTextParagLineStart *KoTextFormatterCore::koBidiReorderLine(
     KoTextParag * /*parag*/, KoTextString *text, KoTextParagLineStart *line,
     KoTextStringChar *startChar, KoTextStringChar *lastChar, int align, int space )
 {
+    // This comes from Qt (3.3.x) but seems wrong: the last space is where we draw
+    // the "end of paragraph" sign, so it needs to be correctly positioned too.
+#if 0
+    // ignore white space at the end of the line.
+    int endSpaces = 0;
+    while ( lastChar > startChar && lastChar->whiteSpace ) {
+        space += lastChar->format()->width( ' ' );
+        --lastChar;
+        ++endSpaces;
+    }
+#endif
+
     int start = (startChar - &text->at(0));
     int last = (lastChar - &text->at(0) );
 #ifdef DEBUG_FORMATTER
